/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
import javax.swing.*;
import gvid.ui.status.*;
import gvid.ui.splitscrollpane.*;
import gvid.document.*;
import gvid.ui.editor.*;
import java.awt.*;
import java.awt.event.*;
import gvid.command.*;
import gvid.Util;

public class ViDApplet 
	extends javax.swing.JApplet 
	implements KeyListener {
	private static final int CTRL = InputEvent.CTRL_MASK;
	private static final int ALT = InputEvent.ALT_MASK;
	private static final int SHIFT = InputEvent.SHIFT_MASK;

	private StatusPanel m_status = null;
	private SourceCodeView m_view = null;
	private SplitScrollPane m_splitPane = null;
	private SourceCodeModel m_document = null;
	public void init() {
		try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
			LookAndFeel laf = new com.sun.java.swing.plaf.windows.WindowsLookAndFeel() {
				public boolean isNativeLookAndFeel() {
					return true;
				}
			};
			UIManager.setLookAndFeel(laf);
		} catch( Exception e ) {
			System.err.println(e);
		}

		setForeground(Color.lightGray);
		setBackground(Color.lightGray);

		gvid.util.CursorManager.getInstance().loadCursors(this);

		m_status = new StatusPanel();
		m_view = new SourceCodeView();
		m_splitPane = new SplitScrollPane();;
		m_document = new SourceCodeModel();

		m_view.setModel(m_document);
		m_splitPane.setView(m_view);
		m_status.setStatusSource(m_view.getStatusSource());

		buildCommandMap();
		Util.recurseAddKeyListener(this, this);

		getContentPane().add(m_splitPane, BorderLayout.CENTER);
		getContentPane().add(m_status, BorderLayout.SOUTH);

		m_splitPane.getActiveView().asComponent().requestFocus();
	}
///////////////////////////////////////////////////////////////////////////////
//	KeyListener
	public void keyTyped(KeyEvent e) {
	}

	private static final int MODMASK = InputEvent.CTRL_MASK|InputEvent.ALT_MASK|InputEvent.SHIFT_MASK;
	public void keyPressed(KeyEvent e) {
		CommandKey key = new CommandKey(e.getKeyCode(), e.getModifiers()&MODMASK);
		CommandId cmdId = (CommandId)m_commandMap.get(key);
		if( cmdId != null ) {
			SourceCodeView view = (SourceCodeView)(m_splitPane.getActiveView().asComponent());
			view.getEditCommands().executeCommand(cmdId.getCmd());
			e.consume();
		}
	}

	public void keyReleased(KeyEvent e) {
	}
//	KeyListener
///////////////////////////////////////////////////////////////////////////////

	java.util.HashMap m_commandMap = new java.util.HashMap();

	private void buildCommandMap() {
		m_commandMap.put( new CommandKey(KeyEvent.VK_LEFT, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.CHARLEFT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_LEFT, CTRL),				new CommandId( CommandBase.EDITGROUP, EditCommands.WORDLEFT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_LEFT, CTRL|ALT),			new CommandId( CommandBase.EDITGROUP, EditCommands.SENTENCELEFT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_LEFT, ALT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLLEFT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_LEFT, SHIFT),				new CommandId( CommandBase.EDITGROUP, EditCommands.CHARLEFTEXTEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_LEFT, CTRL|SHIFT),			new CommandId( CommandBase.EDITGROUP, EditCommands.WORDLEFTEXTEND) );

		m_commandMap.put( new CommandKey(KeyEvent.VK_HOME, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.HOME) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_HOME, ALT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWLEFTEDGE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_HOME, CTRL),				new CommandId( CommandBase.EDITGROUP, EditCommands.DOCUMENTSTART) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_HOME, CTRL|ALT),			new CommandId( CommandBase.EDITGROUP, EditCommands.HOMEBRIEF) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_HOME, SHIFT),				new CommandId( CommandBase.EDITGROUP, EditCommands.HOMEEXTEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_HOME, CTRL|SHIFT),			new CommandId( CommandBase.EDITGROUP, EditCommands.DOCUMENTSTARTEXTEND) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_BACK_QUOTE, CTRL),			new CommandId( CommandBase.EDITGROUP, EditCommands.GOTOINDENTATION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_BACK_QUOTE, CTRL|SHIFT),	new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTSWAPANCHOR) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_RIGHT, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.CHARRIGHT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_RIGHT, CTRL), 				new CommandId( CommandBase.EDITGROUP, EditCommands.WORDRIGHT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_RIGHT, CTRL|ALT),			new CommandId( CommandBase.EDITGROUP, EditCommands.SENTENCERIGHT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_RIGHT, ALT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLRIGHT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_RIGHT, SHIFT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.CHARRIGHTEXTEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_RIGHT, CTRL|SHIFT), 		new CommandId( CommandBase.EDITGROUP, EditCommands.WORDRIGHTEXTEND) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_END, 0), 					new CommandId( CommandBase.EDITGROUP, EditCommands.LINEEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_END, ALT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWRIGHTEDGE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_END, CTRL), 				new CommandId( CommandBase.EDITGROUP, EditCommands.DOCUMENTEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_END, CTRL|ALT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.ENDBRIEF) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_END, CTRL|SHIFT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.DOCUMENTENDEXTEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_END, SHIFT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.LINEENDEXTEND) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_UP, 0), 					new CommandId( CommandBase.EDITGROUP, EditCommands.LINEUP) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_UP, CTRL), 				new CommandId( CommandBase.EDITGROUP, EditCommands.PARAUP) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_UP, ALT), 					new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLDOWN) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_UP, SHIFT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.LINEUPEXTEND) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_UP, 0), 				new CommandId( CommandBase.EDITGROUP, EditCommands.PAGEUP) );		
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_UP, ALT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSTART) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_UP, CTRL), 			new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLTOBOTTOM) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_UP, SHIFT),			new CommandId( CommandBase.EDITGROUP, EditCommands.PAGEUPEXTEND) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_DOWN, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.LINEDOWN) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_DOWN, CTRL),				new CommandId( CommandBase.EDITGROUP, EditCommands.PARADOWN) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_DOWN, ALT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLUP) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_DOWN, SHIFT),				new CommandId( CommandBase.EDITGROUP, EditCommands.LINEDOWNEXTEND) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_DOWN, 0),				new CommandId( CommandBase.EDITGROUP, EditCommands.PAGEDOWN) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_DOWN, ALT),			new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_DOWN, CTRL),			new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLTOTOP) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_DOWN, SHIFT),			new CommandId( CommandBase.EDITGROUP, EditCommands.PAGEDOWNEXTEND) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_PERIOD, CTRL),				new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLTOCENTER) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_INSERT, 0), 				new CommandId( CommandBase.EDITGROUP, EditCommands.EDITTOGGLEOVERTYPE) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_BACK_SLASH, CTRL|SHIFT),	new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTPARA) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_8, CTRL|SHIFT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.TOGGLEVIEWWHITESPACE) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_A, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTALL) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_C, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.COPY) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_U, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTIONLOWERCASE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_U, CTRL|SHIFT),			new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTIONUPPERCASE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_U, ALT),					new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTIONTOGGLECASE) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_V, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.PASTE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_X, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.CUT) );

		m_commandMap.put( new CommandKey(KeyEvent.VK_Y, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.REDO) );
//		m_commandMap.put( new CommandKey(KeyEvent.VK_Y, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.REDOCHANGES) );

		m_commandMap.put( new CommandKey(KeyEvent.VK_Z, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.UNDO) );
//		m_commandMap.put( new CommandKey(KeyEvent.VK_Z, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.UNDOCHANGES) );
				
				
//		m_commandMap.put( new CommandKey(KeyEvent., 0), 						new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTLINE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, 0), 				new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, SHIFT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, CTRL), 			new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, ALT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, SHIFT|CTRL), 		new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, SHIFT|ALT), 		new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, CTRL|ALT), 		new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, SHIFT|CTRL|ALT), 	new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );

		// editing commands
		m_commandMap.put( new CommandKey(KeyEvent.VK_DELETE, 0), 				new CommandId( CommandBase.EDITGROUP, EditCommands.DELETE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_DELETE, SHIFT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.DELETE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_DELETE, CTRL),				new CommandId( CommandBase.EDITGROUP, EditCommands.WORDDELETETOEND) );

		m_commandMap.put( new CommandKey(KeyEvent.VK_BACK_SPACE, 0),			new CommandId( CommandBase.EDITGROUP, EditCommands.DELETEBACK) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_BACK_SPACE, SHIFT),		new CommandId( CommandBase.EDITGROUP, EditCommands.DELETEBACK) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_BACK_SPACE, CTRL),			new CommandId( CommandBase.EDITGROUP, EditCommands.WORDDELETETOBEGINNING) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_TAB, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTIONINDENT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_TAB, SHIFT),				new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTIONUNINDENT) );
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_ENTER, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.SPLITLINE));
		m_commandMap.put( new CommandKey(KeyEvent.VK_ENTER, SHIFT),				new CommandId( CommandBase.EDITGROUP, EditCommands.SPLITLINE));
				
		m_commandMap.put( new CommandKey(KeyEvent.VK_F, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.FIND));
		m_commandMap.put( new CommandKey(KeyEvent.VK_F3, ALT),					new CommandId( CommandBase.EDITGROUP, EditCommands.FIND));
		m_commandMap.put( new CommandKey(KeyEvent.VK_F3, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.FINDNEXT));
		m_commandMap.put( new CommandKey(KeyEvent.VK_F3, SHIFT),				new CommandId( CommandBase.EDITGROUP, EditCommands.FINDPREV));
		m_commandMap.put( new CommandKey(KeyEvent.VK_F3, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.FINDSELECTIONNEXT));
		m_commandMap.put( new CommandKey(KeyEvent.VK_F3, CTRL|SHIFT),			new CommandId( CommandBase.EDITGROUP, EditCommands.FINDSELECTIONPREV));
	}
}