/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.event.*;
import java.util.*;

public abstract class Util {
	private static Hashtable m_dropTargets = new Hashtable();
	
	public static void recurseCreateDropTarget( Container c, DropTargetListener l1, ContainerListener l2 ) {
		m_dropTargets.put(c, new DropTarget(c, l1));
		c.addContainerListener(l2);
			
		Component[] children = c.getComponents();
		for( int i=0; i<children.length; i++ ) {
			if( (children[i] instanceof Container) && !( children[i] instanceof DropTargetListener )) {
				recurseCreateDropTarget((Container)children[i], l1, l2);
			}
		}
//		System.err.println("Adding drop target.");
	}
	
	public static void recurseDestroyDropTarget( Container c, DropTargetListener l1, ContainerListener l2 ) {
		c.removeContainerListener(l2);
		DropTarget target = (DropTarget)m_dropTargets.remove(c);
		if( target != null ) {
			target.removeDropTargetListener(l1);
		}
			
		Component[] children = c.getComponents();
		for( int i=0; i<children.length; i++ ) {
			if( ( children[i] instanceof Container ) && !( children[i] instanceof DropTargetListener )) {
				recurseDestroyDropTarget((Container)children[i], l1, l2);
			}
		}
	}
	
	public static class RecurseDropTargetComponentListener 
		implements ContainerListener {
		DropTargetListener m_root = null;
		public RecurseDropTargetComponentListener(DropTargetListener root) {
			m_root = root;
		}
		
    	public void componentAdded(ContainerEvent e) {
			Component c = e.getChild();
			if( (c instanceof Container) && !( c instanceof DropTargetListener )) {
				Util.recurseCreateDropTarget((Container)c, m_root, this);
			}
    	}

    	public void componentRemoved(ContainerEvent e) {
			Component c = e.getChild();
			if( (c instanceof Container) && !( c instanceof DropTargetListener )) {
				Util.recurseDestroyDropTarget((Container)c, m_root, this);
			}
    	}
	}
// Global KeyListener Support	
	public static void recurseAddKeyListener( Component c, KeyListener kl ) {
		recurseAddKeyListener(c, kl, null);
	}
	
	private static void recurseAddKeyListener( Component c, KeyListener kl, ContainerListener cl ) {
		if( cl == null ) {
			cl = new RecurseKeyContainerListener(kl);
		}
		c.addKeyListener(kl);
		if( c instanceof Container && !(c instanceof javax.swing.JMenuBar) ) {			
			((Container)c).addContainerListener( cl );
			Component[] children = ((Container)c).getComponents();
			for( int i=0; i<children.length; i++ ) {
				recurseAddKeyListener(children[i], kl, cl);
			}
		}
	}
	
	private static void recurseRemoveKeyListener( Component c, KeyListener kl, ContainerListener cl ) {
		c.removeKeyListener(kl);
		if( c instanceof Container && !(c instanceof javax.swing.JMenuBar) ) {
			((Container)c).removeContainerListener( cl );
			Component[] children = ((Container)c).getComponents();
			for( int i=0; i<children.length; i++ ) {
				recurseRemoveKeyListener(children[i], kl, cl);
			}
		}
	}
	
	public static class RecurseKeyContainerListener 
		implements ContainerListener {
		KeyListener m_l = null;
		public RecurseKeyContainerListener(KeyListener l) {
			m_l = l;
		}
		
    	public void componentAdded(ContainerEvent e) {
    		if( !(e.getChild() instanceof javax.swing.JMenuBar ) ) {
				recurseAddKeyListener(e.getChild(), m_l, this);
    		}
    	}

    	public void componentRemoved(ContainerEvent e) {
    		if( !(e.getChild() instanceof javax.swing.JMenuBar ) ) {
				recurseRemoveKeyListener(e.getChild(), m_l, this);
    		}
    	}
	}
// Global FocusListener Support	
	public static void recurseAddFocusListener( Component c, FocusListener kl ) {
		recurseAddFocusListener(c, kl, null);
	}
	
	private static void recurseAddFocusListener( Component c, FocusListener kl, ContainerListener cl ) {
		if( cl == null ) {
			cl = new RecurseFocusContainerListener(kl);
		}
		c.addFocusListener(kl);
		if( c instanceof Container ) {			
			((Container)c).addContainerListener( cl );
			Component[] children = ((Container)c).getComponents();
			for( int i=0; i<children.length; i++ ) {
				recurseAddFocusListener(children[i], kl, cl);
			}
		}
	}
	
	private static void recurseRemoveFocusListener( Component c, FocusListener kl, ContainerListener cl ) {
		c.removeFocusListener(kl);
		if( c instanceof Container ) {
			((Container)c).removeContainerListener( cl );
			Component[] children = ((Container)c).getComponents();
			for( int i=0; i<children.length; i++ ) {
				recurseRemoveFocusListener(children[i], kl, cl);
			}
		}
	}
	
	public static class RecurseFocusContainerListener 
		implements ContainerListener {
		FocusListener m_l = null;
		public RecurseFocusContainerListener(FocusListener l) {
			m_l = l;
		}
		
    	public void componentAdded(ContainerEvent e) {
			recurseAddFocusListener(e.getChild(), m_l, this);
    	}

    	public void componentRemoved(ContainerEvent e) {
			recurseRemoveFocusListener(e.getChild(), m_l, this);
    	}
	}
// Global Menu Focus Return Support	
	public static void returnMenuFocus( javax.swing.JMenuBar c ) {
		returnFocusMenu(c, new gvid.ui.menu.MenuFocusReturn(), null);
	}
	
	private static void returnFocusMenu( javax.swing.JComponent c, gvid.ui.menu.MenuFocusReturn kl, ContainerListener cl ) {
		if( cl == null ) {
			cl = new MenuFocusContainerListener(kl);
		}
		c.getAccessibleContext().addPropertyChangeListener(kl);
		if( c instanceof Container ) {			
			((Container)c).addContainerListener( cl );
			Component[] children = ((Container)c).getComponents();
			for( int i=0; i<children.length; i++ ) {
				returnFocusMenu((javax.swing.JComponent)children[i], kl, cl);
			}
		}
	}
	
	private static void removeFocusMenu( javax.swing.JComponent c, gvid.ui.menu.MenuFocusReturn kl, ContainerListener cl ) {
		c.getAccessibleContext().removePropertyChangeListener(kl);
		if( c instanceof Container ) {
			((Container)c).removeContainerListener( cl );
			Component[] children = ((Container)c).getComponents();
			for( int i=0; i<children.length; i++ ) {
				removeFocusMenu((javax.swing.JComponent)children[i], kl, cl);
			}
		}
	}
	
	public static class MenuFocusContainerListener 
		implements ContainerListener {
		gvid.ui.menu.MenuFocusReturn m_l = null;
		public MenuFocusContainerListener(gvid.ui.menu.MenuFocusReturn l) {
			m_l = l;
		}
		
    	public void componentAdded(ContainerEvent e) {
			returnFocusMenu((javax.swing.JComponent)e.getChild(), m_l, this);
    	}

    	public void componentRemoved(ContainerEvent e) {
			removeFocusMenu((javax.swing.JComponent)e.getChild(), m_l, this);
    	}
	}
}