/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.command;

import gvid.undo.*;

public interface EditCommands extends CommandBase {
    public static final int REDO					= 0;
    public static final int REDOCHANGES 			= 1;
    public static final int UNDO					= 2;
    public static final int UNDOCHANGES 			= 3;
    public static final int CHARLEFT				= 4;
    public static final int WORDLEFT				= 5;
    public static final int SENTENCELEFT			= 6;
    public static final int HOME					= 7;
    public static final int WINDOWLEFTEDGE			= 8;
    public static final int GOTOINDENTATION 		= 9;
    public static final int CHARRIGHT				= 10;
    public static final int WORDRIGHT				= 11;
    public static final int SENTENCERIGHT			= 12;
    public static final int LINEEND 				= 13;
    public static final int WINDOWRIGHTEDGE 		= 14;
    public static final int LINEUP					= 15;
    public static final int PARAUP					= 16;
    public static final int PAGEUP					= 17;
    public static final int DOCUMENTSTART			= 18;
    public static final int WINDOWSTART 			= 19;
    public static final int HOMEBRIEF				= 20;
    public static final int LINEDOWN				= 21;
    public static final int PARADOWN 				= 22;
    public static final int PAGEDOWN 				= 23;
    public static final int DOCUMENTEND 			= 24;
    public static final int WINDOWEND				= 25;
    public static final int ENDBRIEF				= 26;
    public static final int WINDOWSCROLLUP			= 27;
    public static final int WINDOWSCROLLDOWN		= 28;
    public static final int WINDOWSCROLLLEFT		= 29;
    public static final int WINDOWSCROLLRIGHT		= 30;
    public static final int WINDOWSCROLLTOTOP		= 31;
    public static final int WINDOWSCROLLTOBOTTOM	= 32;
    public static final int WINDOWSCROLLTOCENTER	= 33;
    public static final int EDITTOGGLEOVERTYPE		= 34;
    public static final int TOGGLEVIEWWHITESPACE	= 35;
    public static final int CHARLEFTEXTEND			= 36;
    public static final int CHARRIGHTEXTEND 		= 37;
    public static final int DOCUMENTENDEXTEND		= 38;
    public static final int DOCUMENTSTARTEXTEND 	= 39;
    public static final int HOMEEXTEND				= 40;
    public static final int LINEDOWNEXTEND			= 41;
    public static final int LINEENDEXTEND			= 42;
    public static final int LINEUPEXTEND			= 43;
    public static final int PAGEDOWNEXTEND			= 44;
    public static final int PAGEUPEXTEND			= 45;
    public static final int SELECTALL				= 46;
//	public static final int SELECTLINE				= 47;
    public static final int SELECTPARA				= 48;
    public static final int SELECTSWAPANCHOR		= 49;
    public static final int WORDLEFTEXTEND			= 50;
    public static final int WORDRIGHTEXTEND 		= 51;
	public static final int SPLITLINE				= 52;
    public static final int DELETE					= 53;
    public static final int DELETEBACK				= 54;
    public static final int CUT						= 55;
    public static final int COPY					= 56;
    public static final int PASTE					= 57;
    public static final int WORDDELETETOEND			= 58;
    public static final int WORDDELETETOBEGINNING	= 59;
    public static final int SELECTIONLOWERCASE		= 60;
    public static final int SELECTIONUPPERCASE		= 61;
    public static final int SELECTIONTOGGLECASE		= 62;	
    public static final int SELECTIONINDENT			= 63;
    public static final int SELECTIONUNINDENT		= 64;
    public static final int INDENTTOPREV			= 65;
    public static final int CLEARSELECTION			= 66;
    public static final int FIND					= 67;
    public static final int FINDNEXT				= 68;
    public static final int FINDPREV				= 69;
    public static final int FINDSELECTIONNEXT		= 70;
    public static final int FINDSELECTIONPREV		= 71;
    
//  Undo/Redo
    public void redo();             // Redoes the previously undone action
    public void redoChanges();      // Redoes the last action, ignoring movement commands
    public void undo();             // Undoes the last action
    public void undoChanges();      // Undoes the last action, ignoring movement commands
    
//  Move Commands
    public UndoCommand charLeft();         // Moves the cursor one character to the left
    public UndoCommand wordLeft();         // moves back one word
    public UndoCommand sentenceLeft();     // Moves to the beginning of the previous sentence
    public UndoCommand home();             // Move to either the start of the current line, or the start of the text on that line
    public UndoCommand windowLeftEdge();   // Moves to the left edge of the window
    public UndoCommand goToIndentation();  // Moves to the end of the indentation

    public UndoCommand charRight();        // Moves the cursor one character to the right    
    public UndoCommand wordRight();        // Moves forward one word    
    public UndoCommand sentenceRight();    // Moves to the beginning of the next sentence
    public UndoCommand lineEnd();          // Moves to the end of the current line
    public UndoCommand windowRightEdge();  // Moves to the right edge of the window
        
    public UndoCommand lineUp();           // Moves the cursor up one line
    
    public UndoCommand paraUp();           // Moves to the beginning of the previous paragraph
    public UndoCommand pageUp();           // Moves the cursor up one page
    public UndoCommand documentStart();    // Moves to the beginning of the document
    public UndoCommand windowStart();      // Moves to the top of the text window
    public UndoCommand homeBrief();        // Moves to the start of the current line, start of the text window, or start of the file
    
    public UndoCommand lineDown();         // Moves the cursor down one line
    public UndoCommand paraDown();         // Moves to the begining of the next paragraph
    public UndoCommand pageDown();         // Moves the cursor down one page
    public UndoCommand documentEnd();      // Moves to the end of the document
    public UndoCommand windowEnd();        // Moves to the bottom of the text window
    public UndoCommand endBrief();         // Moves to the end of the current line, bottom of the text window, or end of the file

//  Scroll Commands
    public UndoCommand windowScrollUp();
    public UndoCommand windowScrollDown();
    public UndoCommand windowScrollLeft();
    public UndoCommand windowScrollRight();
    public UndoCommand windowScrollToTop();
    public UndoCommand windowScrollToBottom();
    public UndoCommand windowScrollToCenter();
    
//  Mode Commands
    public UndoCommand editToggleOvertype();// Toggles between inserting and replacing text.
    public UndoCommand toggleViewWhitespace();// shows or hides tab characters
    
//  Selection Commands
    public UndoCommand charLeftExtend();   // Extends the selection one character to the left
    public UndoCommand charRightExtend();  // Extends the selection one character to the left
    public UndoCommand documentEndExtend();// Extends the selection to the end of the document   
    public UndoCommand documentStartExtend();// Extends the selection to the beginning of the document
    public UndoCommand homeExtend();       // Extends the selection to either the start of the current line, or the start of the text on that line
    public UndoCommand lineDownExtend();   // Extends the selection down one line
    public UndoCommand lineEndExtend();    // Extends the selection to the end of the current line
    public UndoCommand lineUpExtend();     // Extends the selection up one line
    public UndoCommand pageDownExtend();   // Extends the selection down one page
    public UndoCommand pageUpExtend();     // Extends the selection up one page
    public UndoCommand selectAll();        // Selects the entire document
//    public UndoCommand selectLine();       // selects lines of text
    public UndoCommand selectPara();		// Selects the current paragraph
    public UndoCommand selectSwapAnchor();	// Swaps the anchor and the cursor in the current selection
    public UndoCommand wordLeftExtend();       // Extends the selection back one word
    public UndoCommand wordRightExtend();      // Extends the selection forward one word.
    
// Editing Commands
	public UndoCommand splitLine();
    public UndoCommand delete();           // Deletes the selection or right character
    public UndoCommand deleteBack();       // Deletes the selection, or if there is no selection, the character to the left of the cursor
    public UndoCommand copy();             // Copies the selection to the clipboard
    public UndoCommand cut();              // Cuts the selection to the clipboard
    public UndoCommand paste();            // Inserts clipboard contents at the insertion point
    public UndoCommand wordDeleteToEnd();      // Deletes a word to the right
    public UndoCommand wordDeleteToBeginning();// Deletes a word to the left
    public UndoCommand selectionLowercase();// Makes the selection all lowercase
    public UndoCommand selectionUppercase();// Makes the selection all uppercase
    public UndoCommand selectionTogglecase();// Toggles the case of the selection
    
    public UndoCommand selectionIndent();	// indents currently selected lines
    public UndoCommand selectionUnindent(); // unidents currently selected lines
    
    public UndoCommand indentToPrev();		// indent the current line to position of the previous line
    public UndoCommand clearSelection();	// clears the selection

	public UndoCommand find();
	public UndoCommand findNext();			// Finds the next occurrence of the specified text
	public UndoCommand findPrev();			// Finds the previous occurrence of the specified text
	public UndoCommand findSelectionNext();	// Sets the specified text to the current selection and does findNext
	public UndoCommand findSelectionPrev();	// Sets the specified text to the current selection and does findPrev
/*    
//  Move Commands
    public UndoCommand goTo(Point view);    // Moves to a specified line
    
//  Mode Commands
    public UndoCommand findToggleCaseSensitivity();	// Toggles case sensitive matching
    public UndoCommand findToggleMatchWord();		// Toggles whole word matching
    public UndoCommand findToggleAllDocs();			// Toggles searching across all open documents
    public UndoCommand findToggleWrap();			// Toggles searching wrapping from end ot beginning

	
	findInFiles();
	findRegExpr();
	findRefExprPrev();
	findToggleRegExpr();
//  Edit Commands

	charTranspose()
	conditionalDown() - finds the next matched preprocessor condition
	conditionalDownExtend()
	conditionalUp() - finds the next matched preprocessor condition
	conditionalUpExtend()
	cutAppendNext()
	cutSelection()
	deleteBlankLines()
	deleteHorizontalSpace()
	findTool()
	goToMatchBrace()
	goToMatchBraceExtend()
	indentSelectionToPrev()
	insertQuoted()
	levelCutToEnd()
	levelCutToStart()
	levelDown()
	levelUp()
	lineCut()
	lineDelete()
	lineDeleteToEnd()
	lineDeleteToStart()
	lineIndent()
	lineOpenAbove()
	lineOpenBelow()
	lineStart()
	lineTranspose()
	lineUnindent()
	searchIncremental()
	searchIncrementalBack()
	selectChar()
	selectCharInclusive()
	selectColumn()
	selectHighlight()
	selectionFormat()
	selectionTabify()
	selectionUntabify()
	sentenceCut()
	wordCapitalize()
	wordDeleteToEnd()
	wordDeleteToStart()
	wordLowerCase()
	wordToggleCase()
	wordTranspose()
	wordUpperCase()
*/
}