/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.project;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.beans.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.text.*;
import gvid.Util;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import gvid.ui.MainFrame;

public class FilelistPanel 
	extends 	javax.swing.JPanel 
	implements	TableModel,
				DropTargetListener,
				MouseListener,
				KeyListener,
				ProjectListener {
	public FilelistPanel() {
		m_table.addMouseListener(this);
		m_table.getTableHeader().addMouseListener(this);
		m_table.getTableHeader().setReorderingAllowed(false);
		m_table.setColumnSelectionAllowed(false);
		m_table.setIntercellSpacing(new Dimension(0,0));
		m_table.setShowGrid(false);
		m_table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		
		setLayout( new BorderLayout() );
		m_table.setModel(this);
		m_scroll = new JScrollPane(m_table);
		add( m_scroll, BorderLayout.CENTER );
		Util.recurseCreateDropTarget(this, this, new Util.RecurseDropTargetComponentListener(this));
		m_table.addKeyListener(this);
	}
	
	JTable m_table = new JTable();
	JScrollPane m_scroll = null;
	ProjectModel m_project = null;
	
	public void setProjectModel( ProjectModel model ) {
		synchronized( m_listeners ) {
			if( m_project != null ) {
				m_project.removeProjectListener(this);
			}
			m_project = model;
			if( m_project != null ) {
				m_project.addProjectListener(this);
			}
			clearSort();
			sortColumn(0);
			updateTable();
		}
	}
	
    public int getRowCount() {
    	int rc = 0;
    	if( m_project != null ) {
    		rc = m_project.getFileList().size();
    	}
    	return rc;
    }

	private static final String[] m_columnNames = {
		"Name",
		"Folder", 
		"Type",		
		"Modified"
	};
    public int getColumnCount() {
    	int rc = m_columnNames.length;
    	return rc;
    }
    
    public String getColumnName(int columnIndex) {
    	String rc = m_columnNames[columnIndex];
    	return rc;
    }
    
    public Class getColumnClass(int columnIndex) {
    	Class rc = String.class;
    	return rc;
    }
    
    public boolean isCellEditable(int rowIndex, int columnIndex) {
    	boolean rc = false;
    	return rc;
    }
    
    public Object getValueAt(int rowIndex, int columnIndex) {
    	String rc = "";
    	File file = (File)m_project.getFileList().get(rowIndex);
    	switch( columnIndex ) {
    		case 0: {
    			rc = file.getName();
    		} break;
    		case 1: {
    			rc = file.getParent();
    		} break;
    		case 2: {
    			rc = "";
    		} break;
    		case 3: {
    			long mod = file.lastModified();
    			if( mod != 0 ) {
    				rc = DateFormat.getDateTimeInstance().format(new Date(mod));
    			}
    		} break;
    	}
    	return rc;
    }
    
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
    	return;
    }
    
    Vector m_listeners = new Vector();
    public void addTableModelListener(TableModelListener l) {
    	synchronized( m_listeners ) {
    		m_listeners.add(l);
    	}
    }
    
    public void removeTableModelListener(TableModelListener l) {
    	synchronized( m_listeners ) {
    		m_listeners.remove(l);
    	}
    }    
    
    private void updateTable() {
    	synchronized( m_listeners ) {
    		TableModelEvent event = new TableModelEvent(this);
    		Object[] list = m_listeners.toArray();
    		for( int i = 0; i < list.length; i++ ) {
    			((TableModelListener)list[i]).tableChanged(event);
    		}
    	}
    }
    
    private int m_columnSort = -1;
    private boolean m_bForwardSort = true;
    private Comparator[] m_comparators = {
    	new FilenameComparator(),
    	new FolderComparator(),
    	new FilenameComparator(),
    	new LastChangedComparator()
    };
    private InverseComparator m_inverse = new InverseComparator();
    private void sortColumn( int index ) {
    	if( m_project != null ) {
    		if( index == m_columnSort ) {
    			m_bForwardSort = !m_bForwardSort;
    		}
    		m_columnSort = index;
    		if( m_bForwardSort ) {
    			Collections.sort( m_project.getFileList(), m_comparators[index] );
    		} else {
    			m_inverse.setComparator( m_comparators[index] );
    			Collections.sort( m_project.getFileList(), m_inverse );
    		}
    		updateTable();
    	}
    }
    private void clearSort() {
		m_columnSort = -1;
		m_bForwardSort = true;
    }
///////////////////////////////////////////////////////////////////////////////
//	DropTargetListener	
	DropTarget m_dropTarget = new DropTarget(this, this);

	public void dragEnter(DropTargetDragEvent dtde) {
	}
	
	public void dragOver(DropTargetDragEvent dtde) {
		if( !dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor) ) {
			dtde.rejectDrag();
		}
	}
	
	public void dropActionChanged(DropTargetDragEvent dtde) {
	}
	
	public void dragExit(DropTargetEvent dte) {
	}
	
	public void drop(DropTargetDropEvent dtde) {
		if( m_project == null ) {
			if( !MainFrame.getMainFrame().newProject() ) {
				dtde.dropComplete(true);
			}
		}
		
		if( m_project != null ) {
			Transferable trans = dtde.getTransferable();
			if( dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor) ) {
				try {
					dtde.acceptDrop(DnDConstants.ACTION_COPY);
					java.util.List files = (java.util.List)trans.getTransferData(DataFlavor.javaFileListFlavor);
					Iterator iter = files.iterator();
					java.util.List fileList = m_project.getFileList();
					while( iter.hasNext() ) {
						File f = (File)iter.next();
						if( !f.isDirectory() && !fileList.contains(f) ) {
							fileList.add(f);
						}
					}
					updateTable();
				} catch( IOException e ) {
					System.err.println(e);
				} catch( UnsupportedFlavorException e ) {
					System.err.println(e);
				} finally {
					dtde.dropComplete(true);
				}
			}
		}
	}
//	DropTargetListener	
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	MouseListener	
    public void mouseClicked(MouseEvent e) {
    	if( e.getSource() == m_table.getTableHeader() ) {
    		sortColumn(m_table.getTableHeader().columnAtPoint(e.getPoint()));
    	} else if( e.getSource() == m_table && e.getClickCount()==2 ) {
    		int row = m_table.rowAtPoint(e.getPoint());
    		File file = (File)m_project.getFileList().get(row);
    		MainFrame.getMainFrame().openFile(file);
    	}
    }
    
    public void mousePressed(MouseEvent e) {
    }
    
    public void mouseReleased(MouseEvent e) {
    }
    
    public void mouseEntered(MouseEvent e) {
    }
    
    public void mouseExited(MouseEvent e) {
    }

//	MouseListener	
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	KeyListener	
    public void keyTyped(KeyEvent e) {
    }

    public void keyPressed(KeyEvent e) {
    	if( e.getKeyCode() == KeyEvent.VK_DELETE ) {
    		int[] selRows = m_table.getSelectedRows();
    		if( selRows.length > 0 ) {
    			String message = "Are you sure you want to remove the selected files from the project?";
    			if( selRows.length == 1 ) {
	    			message = "Are you sure you want to remove the selected file from the project?";
    			}
    			if( JOptionPane.showConfirmDialog(this, message, "Remove files", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION ) {
    				ArrayList list = new ArrayList(selRows.length);
    				for( int i=0; i < selRows.length; i++ ) {
    					String filename = m_table.getValueAt(selRows[i],1).toString()+
    						File.separator+
    						m_table.getValueAt(selRows[i],0);
    					File file = new File(filename);
    					list.add(file);
    				}
    				m_project.getFileList().removeAll(list);
    			}
    		}
    	}
    }

    public void keyReleased(KeyEvent e) {
    }
//	KeyListener	
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	ProjectListener	
	public void filesChanged() {
		updateTable();
	}
//	ProjectListener	
///////////////////////////////////////////////////////////////////////////////
	private static class FilenameComparator implements Comparator {
    	public int compare(Object o1, Object o2) { 
    		return ((File)o1).getName().compareTo(((File)o2).getName());
    	}
	}
	private static class FolderComparator implements Comparator {
    	public int compare(Object o1, Object o2) { 
    		return ((File)o1).getParent().compareTo(((File)o2).getParent());
    	}
	}
	private static class LastChangedComparator implements Comparator {
    	public int compare(Object o1, Object o2) { 
    		long diff = (((File)o1).lastModified() - ((File)o2).lastModified());
    		int rc = 0;
    		if( diff < 0 ) {
    			rc = -1;
    		} else if( diff > 0 ) {
    			rc = 1;
    		}
    		return rc;
    	}
	}
	private static class InverseComparator implements Comparator {
		Comparator m_comparator = null;
		public void setComparator( Comparator comparator ) {
			m_comparator = comparator;
		}
    	public int compare(Object o1, Object o2) { 
    		return -m_comparator.compare(o1, o2);
    	}
	}
}