/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.project;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class ProjectSettingsDialog extends javax.swing.JDialog {
	private ProjectSettingsPanel m_settings = new ProjectSettingsPanel();
	
	public ProjectSettingsDialog(Frame parent) {
		super(parent, "Project Settings", true);

		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		getContentPane().setLayout( new BorderLayout() );
		getContentPane().add( m_settings, BorderLayout.CENTER );

		m_okButton.addActionListener( new ActionListener() {
			public void actionPerformed( java.awt.event.ActionEvent evt ) {
				ok();
			}
		});
		
		m_cancelButton.addActionListener( new ActionListener() {
			public void actionPerformed( java.awt.event.ActionEvent evt ) {
				cancel();
			}
		});
		
		m_applyButton.addActionListener( new ActionListener() {
			public void actionPerformed( java.awt.event.ActionEvent evt ) {
				apply();
			}
		});
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout( new GridLayout(1,3) );
		buttonPanel.add( m_okButton );
		buttonPanel.add( m_cancelButton );
		buttonPanel.add( m_applyButton );
		
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);

		doLayout();
		pack();	
	}
	
	private ProjectModel m_project = null;

	private JButton m_okButton = new JButton("Ok");
	private JButton m_cancelButton = new JButton("Cancel");
	private JButton m_applyButton = new JButton("Apply");
	
	
	public void setProject( ProjectModel project ) {
		m_project = project;
	}
	
	public void setVisible( boolean b ) {
		if( b && m_project == null ) {
			System.err.println("ProjectSettingsDialog must have a project set prior to showing the dialog.");
		} else if( b ) {
			m_settings.initSettings( m_project );
		}
		super.setVisible(b);
	}
	
	private void ok() {
		apply();
		cancel();
	}
	
	private void cancel() {
		setVisible(false);
		setProject( null );
	}
	
	private void apply() {
		m_settings.extractSettings( m_project );
	}

}