/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui;

import gvid.command.*;
import java.awt.event.*;

public class MainCommandHandler 
	implements	gvid.command.CommandHandler,
				ContainerListener {
	private static final int CTRL = InputEvent.CTRL_MASK;
	private static final int ALT = InputEvent.ALT_MASK;
	private static final int SHIFT = InputEvent.SHIFT_MASK;
	private MainFrame m_mainFrame = null;

	public MainCommandHandler( MainFrame mainframe ) {
		m_mainFrame = mainframe;
		m_mainFrame.getDesktop().addContainerListener(this);
		buildCommandKeyMap();
	}
	
    public void executeCommand(int group, int cmd) {
    	switch( group ) {
    		case CommandBase.FILEGROUP: {
    			m_mainFrame.getFileCommands().executeCommand(cmd);
    		} break;
    		case CommandBase.EDITGROUP: {
    			javax.swing.JDesktopPane desktop = m_mainFrame.getDesktop();
    			if( desktop.getComponentCount() > 0 ) {
    				java.awt.Component c = desktop.getComponent(0);
    				if( c instanceof InternalTextFrame ) {
    					InternalTextFrame itf = (InternalTextFrame)c;
    					itf.getEditCommands().executeCommand(cmd);
    				}
    			}
    		} break;
    		case CommandBase.VIEWGROUP: {
    			m_mainFrame.getViewCommands().executeCommand(cmd);
    		} break;
    		case CommandBase.PROJECTGROUP: {
    			m_mainFrame.getProjectCommands().executeCommand(cmd);
    		} break;
    		case CommandBase.TOOLGROUP: {
    		} break;
    		case CommandBase.WINDOWGROUP: {
    		} break;
    		case CommandBase.HELPGROUP: {
    		} break;
    	}
    }
    
	boolean[] m_groupFlags = { true, false, true, false, false, true, true };
	public boolean isGroupAvailable( int group ) {
		boolean rc = false;
    	switch( group ) {
    		case CommandBase.FILEGROUP: {
    			rc = true;
    		} break;
    		case CommandBase.EDITGROUP: {
    			javax.swing.JDesktopPane desktop = m_mainFrame.getDesktop();
    			if( desktop.getComponentCount() > 0 ) {
    				java.awt.Component c = desktop.getComponent(0);
    				if( c instanceof InternalTextFrame ) {
    					rc = true;
    				}
    			}
    		} break;
    		case CommandBase.VIEWGROUP: {
    			rc = true;
    		} break;
    		case CommandBase.PROJECTGROUP: {
    			rc = (m_mainFrame.getProject()!=null);
    		} break;
    		case CommandBase.TOOLGROUP: {
    			rc = false;
    		} break;
    		case CommandBase.WINDOWGROUP: {
    			rc = true;
    		} break;
    		case CommandBase.HELPGROUP: {
    			rc = true;
    		} break;
    	}
    	if( rc != m_groupFlags[group] ) {
    		fireGroupChanged(group, rc, m_groupFlags[group]);
    		m_groupFlags[group] = rc;
    	}
    	return rc;
	}

	public boolean isCommandAvailable( int group, int cmd ) {
		boolean rc = false;
    	switch( group ) {
    		case CommandBase.FILEGROUP: {
    			rc = m_mainFrame.getFileCommands().isAvailable(cmd);
    		} break;
    		case CommandBase.EDITGROUP: {
    			javax.swing.JDesktopPane desktop = m_mainFrame.getDesktop();
    			if( desktop.getComponentCount() > 0 ) {
    				java.awt.Component c = desktop.getComponent(0);
    				if( c instanceof InternalTextFrame ) {
    					InternalTextFrame itf = (InternalTextFrame)c;
    					itf.getEditCommands().isAvailable(cmd);
    				}
    			}
    		} break;
    		case CommandBase.VIEWGROUP: {
    		} break;
    		case CommandBase.TOOLGROUP: {
    		} break;
    		case CommandBase.WINDOWGROUP: {
    		} break;
    		case CommandBase.HELPGROUP: {
    		} break;
    	}
    	return rc;
	}
 
	java.util.HashMap m_commandMap = new java.util.HashMap();
    public java.util.HashMap getCommandKeyMap() {
    	return m_commandMap;
    }
    
    java.util.Vector m_groupListeners = new java.util.Vector();
	public void addCommandGroupListener( CommandGroupListener l ) {
		m_groupListeners.addElement(l);
	}
	
	public void removeCommandGroupListener( CommandGroupListener l ) {
		m_groupListeners.removeElement(l);
	}   
	
	public gvid.ui.status.StatusSource getStatusSource() {
		return m_mainFrame.getStatusSource();
	}
	
	private void fireGroupChanged( int group, boolean newValue, boolean oldValue ) {
		java.util.Enumeration enum = m_groupListeners.elements();
		while( enum.hasMoreElements() ) {
			CommandGroupListener l = (CommandGroupListener)enum.nextElement();
			l.groupAvailabilityChanged(group, newValue, oldValue);
		}
	}
    
    private void buildCommandKeyMap() {
    	// File Group
		m_commandMap.put( new CommandKey(KeyEvent.VK_F4, ALT),			new CommandId( CommandBase.FILEGROUP, FileCommands.APPLICATIONEXIT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_F4, CTRL),			new CommandId( CommandBase.FILEGROUP, FileCommands.FILECLOSE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_F4, CTRL|ALT),		new CommandId( CommandBase.FILEGROUP, FileCommands.FILESAVEALLEXIT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_O, CTRL),			new CommandId( CommandBase.FILEGROUP, FileCommands.FILEOPEN) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_P, CTRL),			new CommandId( CommandBase.FILEGROUP, FileCommands.FILEPRINT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_P, CTRL|ALT),		new CommandId( CommandBase.FILEGROUP, FileCommands.FILEPRINTPAGESETUP) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_S, CTRL),			new CommandId( CommandBase.FILEGROUP, FileCommands.FILESAVE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_S, CTRL|SHIFT),	new CommandId( CommandBase.FILEGROUP, FileCommands.FILESAVEALL) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_S, CTRL|ALT),		new CommandId( CommandBase.FILEGROUP, FileCommands.FILESAVEAS) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_N, CTRL|ALT),		new CommandId( CommandBase.FILEGROUP, FileCommands.NEWITEM) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_N, CTRL),			new CommandId( CommandBase.FILEGROUP, FileCommands.NEWTEXT) );
//		m_commandMap.put( new CommandKey(KeyEvent.VK_TAB, CTRL),		new CommandId( CommandBase.FILEGROUP, FileCommands.WINDOWNEXT) );
//		m_commandMap.put( new CommandKey(KeyEvent.VK_TAB, CTRL|SHIFT),	new CommandId( CommandBase.FILEGROUP, FileCommands.WINDOWPREV) );
//		m_commandMap.put( new CommandKey(KeyEvent., 0),					new CommandId( CommandBase.FILEGROUP, FileCommands.FILEGOTO) );
//		m_commandMap.put( new CommandKey(KeyEvent., 0),					new CommandId( CommandBase.FILEGROUP, FileCommands.WORKSPACECLOSE) );
//		m_commandMap.put( new CommandKey(KeyEvent., 0),					new CommandId( CommandBase.FILEGROUP, FileCommands.WORKSPACEOPEN) );
//		m_commandMap.put( new CommandKey(KeyEvent., 0),					new CommandId( CommandBase.FILEGROUP, FileCommands.WORKSPACESAVE) );
//		m_commandMap.put( new CommandKey(KeyEvent., 0),					new CommandId( CommandBase.FILEGROUP, FileCommands.WINDOWCLOSEALL) );
//		m_commandMap.put( new CommandKey(KeyEvent., 0),					new CommandId( CommandBase.FILEGROUP, FileCommands.WINDOWCASCADE) );
//		m_commandMap.put( new CommandKey(KeyEvent., 0),					new CommandId( CommandBase.FILEGROUP, FileCommands.WINDOWTILEHORIZONTAL) );
//		m_commandMap.put( new CommandKey(KeyEvent., 0),					new CommandId( CommandBase.FILEGROUP, FileCommands.WINDOWTILEVERTICAL) );
//		m_commandMap.put( new CommandKey(KeyEvent., 0),					new CommandId( CommandBase.FILEGROUP, FileCommands.WINDOWTILE) );
//		m_commandMap.put( new CommandKey(KeyEvent., 0),					new CommandId( CommandBase.FILEGROUP, FileCommands.WINDOWLIST) );
    	// Edit Group
		m_commandMap.put( new CommandKey(KeyEvent.VK_LEFT, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.CHARLEFT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_LEFT, CTRL),				new CommandId( CommandBase.EDITGROUP, EditCommands.WORDLEFT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_LEFT, CTRL|ALT),			new CommandId( CommandBase.EDITGROUP, EditCommands.SENTENCELEFT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_LEFT, ALT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLLEFT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_LEFT, SHIFT),				new CommandId( CommandBase.EDITGROUP, EditCommands.CHARLEFTEXTEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_LEFT, CTRL|SHIFT),			new CommandId( CommandBase.EDITGROUP, EditCommands.WORDLEFTEXTEND) );

		m_commandMap.put( new CommandKey(KeyEvent.VK_HOME, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.HOME) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_HOME, ALT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWLEFTEDGE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_HOME, CTRL),				new CommandId( CommandBase.EDITGROUP, EditCommands.DOCUMENTSTART) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_HOME, CTRL|ALT),			new CommandId( CommandBase.EDITGROUP, EditCommands.HOMEBRIEF) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_HOME, SHIFT),				new CommandId( CommandBase.EDITGROUP, EditCommands.HOMEEXTEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_HOME, CTRL|SHIFT),			new CommandId( CommandBase.EDITGROUP, EditCommands.DOCUMENTSTARTEXTEND) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_BACK_QUOTE, CTRL),			new CommandId( CommandBase.EDITGROUP, EditCommands.GOTOINDENTATION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_BACK_QUOTE, CTRL|SHIFT),	new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTSWAPANCHOR) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_RIGHT, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.CHARRIGHT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_RIGHT, CTRL), 				new CommandId( CommandBase.EDITGROUP, EditCommands.WORDRIGHT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_RIGHT, CTRL|ALT),			new CommandId( CommandBase.EDITGROUP, EditCommands.SENTENCERIGHT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_RIGHT, ALT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLRIGHT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_RIGHT, SHIFT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.CHARRIGHTEXTEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_RIGHT, CTRL|SHIFT), 		new CommandId( CommandBase.EDITGROUP, EditCommands.WORDRIGHTEXTEND) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_END, 0), 					new CommandId( CommandBase.EDITGROUP, EditCommands.LINEEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_END, ALT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWRIGHTEDGE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_END, CTRL), 				new CommandId( CommandBase.EDITGROUP, EditCommands.DOCUMENTEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_END, CTRL|ALT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.ENDBRIEF) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_END, CTRL|SHIFT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.DOCUMENTENDEXTEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_END, SHIFT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.LINEENDEXTEND) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_UP, 0), 					new CommandId( CommandBase.EDITGROUP, EditCommands.LINEUP) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_UP, CTRL), 				new CommandId( CommandBase.EDITGROUP, EditCommands.PARAUP) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_UP, ALT), 					new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLDOWN) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_UP, SHIFT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.LINEUPEXTEND) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_UP, 0), 				new CommandId( CommandBase.EDITGROUP, EditCommands.PAGEUP) );		
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_UP, ALT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSTART) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_UP, CTRL), 			new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLTOBOTTOM) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_UP, SHIFT),			new CommandId( CommandBase.EDITGROUP, EditCommands.PAGEUPEXTEND) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_DOWN, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.LINEDOWN) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_DOWN, CTRL),				new CommandId( CommandBase.EDITGROUP, EditCommands.PARADOWN) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_DOWN, ALT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLUP) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_DOWN, SHIFT),				new CommandId( CommandBase.EDITGROUP, EditCommands.LINEDOWNEXTEND) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_DOWN, 0),				new CommandId( CommandBase.EDITGROUP, EditCommands.PAGEDOWN) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_DOWN, ALT),			new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWEND) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_DOWN, CTRL),			new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLTOTOP) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_PAGE_DOWN, SHIFT),			new CommandId( CommandBase.EDITGROUP, EditCommands.PAGEDOWNEXTEND) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_PERIOD, CTRL),				new CommandId( CommandBase.EDITGROUP, EditCommands.WINDOWSCROLLTOCENTER) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_INSERT, 0), 				new CommandId( CommandBase.EDITGROUP, EditCommands.EDITTOGGLEOVERTYPE) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_BACK_SLASH, CTRL|SHIFT),	new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTPARA) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_8, CTRL|SHIFT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.TOGGLEVIEWWHITESPACE) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_A, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTALL) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_C, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.COPY) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_U, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTIONLOWERCASE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_U, CTRL|SHIFT),			new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTIONUPPERCASE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_U, ALT),					new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTIONTOGGLECASE) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_V, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.PASTE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_X, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.CUT) );

		m_commandMap.put( new CommandKey(KeyEvent.VK_Y, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.REDO) );
//		m_commandMap.put( new CommandKey(KeyEvent.VK_Y, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.REDOCHANGES) );

		m_commandMap.put( new CommandKey(KeyEvent.VK_Z, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.UNDO) );
//		m_commandMap.put( new CommandKey(KeyEvent.VK_Z, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.UNDOCHANGES) );
		
		
//		m_commandMap.put( new CommandKey(KeyEvent., 0), new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTLINE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, 0), 				new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, SHIFT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, CTRL), 			new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, ALT), 				new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, SHIFT|CTRL), 		new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, SHIFT|ALT), 		new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, CTRL|ALT), 		new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_ESCAPE, SHIFT|CTRL|ALT), 	new CommandId( CommandBase.EDITGROUP, EditCommands.CLEARSELECTION) );

// editing commands
		m_commandMap.put( new CommandKey(KeyEvent.VK_DELETE, 0), 				new CommandId( CommandBase.EDITGROUP, EditCommands.DELETE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_DELETE, SHIFT), 			new CommandId( CommandBase.EDITGROUP, EditCommands.DELETE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_DELETE, CTRL),				new CommandId( CommandBase.EDITGROUP, EditCommands.WORDDELETETOEND) );

		m_commandMap.put( new CommandKey(KeyEvent.VK_BACK_SPACE, 0),			new CommandId( CommandBase.EDITGROUP, EditCommands.DELETEBACK) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_BACK_SPACE, SHIFT),		new CommandId( CommandBase.EDITGROUP, EditCommands.DELETEBACK) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_BACK_SPACE, CTRL),			new CommandId( CommandBase.EDITGROUP, EditCommands.WORDDELETETOBEGINNING) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_TAB, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTIONINDENT) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_TAB, SHIFT),				new CommandId( CommandBase.EDITGROUP, EditCommands.SELECTIONUNINDENT) );
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_ENTER, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.SPLITLINE));
		m_commandMap.put( new CommandKey(KeyEvent.VK_ENTER, SHIFT),				new CommandId( CommandBase.EDITGROUP, EditCommands.SPLITLINE));
		
		m_commandMap.put( new CommandKey(KeyEvent.VK_F3, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.FIND));
		m_commandMap.put( new CommandKey(KeyEvent.VK_F3, ALT),					new CommandId( CommandBase.EDITGROUP, EditCommands.FIND));
		m_commandMap.put( new CommandKey(KeyEvent.VK_F3, 0),					new CommandId( CommandBase.EDITGROUP, EditCommands.FINDNEXT));
		m_commandMap.put( new CommandKey(KeyEvent.VK_F3, SHIFT),				new CommandId( CommandBase.EDITGROUP, EditCommands.FINDPREV));
		m_commandMap.put( new CommandKey(KeyEvent.VK_F3, CTRL),					new CommandId( CommandBase.EDITGROUP, EditCommands.FINDSELECTIONNEXT));
		m_commandMap.put( new CommandKey(KeyEvent.VK_F3, CTRL|SHIFT),			new CommandId( CommandBase.EDITGROUP, EditCommands.FINDSELECTIONPREV));		
		
// project commands
		m_commandMap.put( new CommandKey(KeyEvent.VK_F7, 0),					new CommandId( CommandBase.PROJECTGROUP, ProjectCommands.BUILD) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_F7, CTRL),					new CommandId( CommandBase.PROJECTGROUP, ProjectCommands.COMPILEFILE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_F5, CTRL),					new CommandId( CommandBase.PROJECTGROUP, ProjectCommands.EXECUTE) );
		m_commandMap.put( new CommandKey(KeyEvent.VK_F5, 0),					new CommandId( CommandBase.PROJECTGROUP, ProjectCommands.DEBUG) );
    }
///////////////////////////////////////////////////////////////////////////////
// ContainerListener
	public void componentAdded(ContainerEvent e) {
		isGroupAvailable(CommandBase.EDITGROUP);
	}

	public void componentRemoved(ContainerEvent e) {
		isGroupAvailable(CommandBase.EDITGROUP);
	}
// ContainerListener
///////////////////////////////////////////////////////////////////////////////
}