/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui;

import gvid.command.*;
import gvid.project.*;

import javax.swing.*;
import java.io.*;
import java.util.*;
import sun.tools.javac.*;

public class MainProjectCommandsImpl 
	implements gvid.command.ProjectCommands {
	MainFrame m_mainFrame = null;
		
	public MainProjectCommandsImpl(MainFrame frame) {
        m_mainFrame = frame;  
    }
	
	public int getCommandGroup() {
		return PROJECTGROUP;
	}	
	
	public boolean isAvailable( int cmd ) {
		boolean rc = false;
		
		// TODO: Implement isAvailable
		rc = true;
		
		return rc;
	}
	
	public void executeCommand(int cmd) {
		switch( cmd ) {
			case PROJECTSETTINGS: {
				projectSettings();
			} break;
			case ADDFILES: {
				addFiles();
			} break;
			case COMPILEFILE: {
				compileFile();
			} break;
			case BUILD: {
				build();
			} break;
			case REBUILDALL: {
				rebuildAll();
			} break;
			case CLEAN: {
				clean();
			} break;
			case EXECUTE: {
				execute();
			} break;
			case DEBUG: {
				debug();
			} break;
			case ATTACHDEBUG: {
				attachDebug();
			} break;
		}
	}
	
    public void toggleProjectView() {
    	m_mainFrame.showProjectWindow(!m_mainFrame.isProjectWindowShowing());
    }

	ProjectSettingsDialog m_settingsDlg = null;
    public void projectSettings() {
    	if( m_settingsDlg == null ) {
    		m_settingsDlg = new ProjectSettingsDialog(MainFrame.getMainFrame());
    	}
    	m_settingsDlg.setProject( MainFrame.getMainFrame().getProject() );
    	m_settingsDlg.setVisible( true );
    }
    
    public void addFiles() {
    }

	public void compileFile() {
		InternalTextFrame topFrame = m_mainFrame.getTopFrame();
		if( topFrame == null ) { return; }
    	File javaFile = topFrame.getFile();
    	String[] filelist = { javaFile.getAbsolutePath() };
    	int result = 0;
    	if( filelist[0].toString().endsWith(".java") ) {
    		result = buildJavaFiles( filelist );
    	} else {
    		result = copyNonJavaFiles( filelist );
    	}
		if( result == Main.EXIT_OK ) {
			System.out.println("Compile successful");
		} else {
			System.out.println("Compile Failed");
		}
	}
	
	public void build() {
		if( m_mainFrame.getProject() != null ) {
			List filelist = m_mainFrame.getProject().getFileList();
			
			// separate the java from the non-java files
			// build the list of java filename strings
			ArrayList nonJavaFiles = new ArrayList();
			ArrayList javaFilenameList = new ArrayList();
			for( int i=0; i<filelist.size(); i++ ) {
				if( !filelist.get(i).toString().endsWith(".java") ) {
					nonJavaFiles.add(filelist.get(i).toString());
				} else {
					File javaFile = (File)filelist.get(i);
					File target = getJavaTargetFile( javaFile );
					if( !target.exists() || target.lastModified() < javaFile.lastModified() ) {
						javaFilenameList.add(filelist.get(i).toString());
					}
				}
			}			
			int rc = 0;
			rc += buildJavaFiles(extractStringArray(javaFilenameList));
			rc += copyNonJavaFiles(extractStringArray(nonJavaFiles));
			if( rc == Main.EXIT_OK ) {
				System.out.println("Build successful");
			} else {
				System.out.println("Build Failed");
			}
		}
	}
	
	public void rebuildAll() {
		if( m_mainFrame.getProject() != null ) {
			List filelist = m_mainFrame.getProject().getFileList();
			
			// separate the java from the non-java files
			// build the list of java filename strings
			ArrayList nonJavaFiles = new ArrayList();
			ArrayList javaFilenameList = new ArrayList();
			for( int i=0; i<filelist.size(); i++ ) {
				if( !filelist.get(i).toString().endsWith(".java") ) {
					nonJavaFiles.add(filelist.get(i).toString());
				} else {
					javaFilenameList.add(filelist.get(i).toString());
				}
			}
			
			int rc = 0;
			rc += buildJavaFiles(extractStringArray(javaFilenameList));
			rc += copyNonJavaFiles(extractStringArray(nonJavaFiles));
			if( rc == Main.EXIT_OK ) {
				System.out.println("Build successful");
			} else {
				System.out.println("Build Failed");
			}
		}
	}
	
	public void clean() {
	}
	
	public void execute() {
	}
	
	public void debug() {
		if( m_mainFrame.getProject() != null ) {
			ArrayList options = new ArrayList();
			buildSourcepathOption(m_mainFrame.getProject(), options);
			options.add("-launch");
			buildClasspathOption(m_mainFrame.getProject(), options);
			options.add(m_mainFrame.getProject().getMainClass());
			StringTokenizer args = new StringTokenizer( m_mainFrame.getProject().getArgs() );
			while( args.hasMoreTokens() ) {
				options.add( args.nextToken() );
			}
			
		}
	}
	
	public void attachDebug() {
	}
	
	private File getJavaTargetFile( File file ) {
		File rc = null;
		if( m_mainFrame.getProject() != null ) {
			if( file.toString().endsWith(".java") ) {
				String className = file.getName().substring(0, file.getName().length()-5)+".class";
				rc = new File( m_mainFrame.getProject().getDestinationRoot(), className );				
			} else {
				rc = new File( m_mainFrame.getProject().getDestinationRoot(), file.getName() );
			}
		}
		return rc;
	}
	
	private int copyNonJavaFiles( String[] filelist ) {
		if( m_mainFrame.getProject() == null ) { return 0; }
		ProjectModel project = m_mainFrame.getProject();
		if( !project.getDestinationRoot().equals(project.getSourceRoot()) ) {
			// TODO : copy files
			System.out.println("Non Java files not copied to the target.");
		}
		return 0;
	}
	
    Main m_javac = new Main(System.out, "javac");
	private int buildJavaFiles( String[] filelist ) {
    	ArrayList options = new ArrayList();
    	buildDebugOption( m_mainFrame.getProject(), options );
    	options.add("-deprecation");
    	buildClasspathOption( m_mainFrame.getProject(), options );
    	buildSourcepathOption( m_mainFrame.getProject(), options );
    	buildDestinationOptions( m_mainFrame.getProject(), options );
    	options.addAll(Arrays.asList(filelist));
		m_javac.compile(extractStringArray(options));
		return m_javac.getExitStatus();
	}
	
	private void buildDebugOption( ProjectModel project, List options ) {
    	if( project.getReleaseType() == ProjectModel.DEBUG ) {
    		options.add( "-g" );
    	} else {
    		options.add("-g:none");
    	}		
	}
	
	private void buildClasspathOption( ProjectModel project, List options ) {
		Object[] classpath = project.getClasspath().toArray();
		StringBuffer classpathStr = new StringBuffer();
		for( int i=0; i<classpath.length; i++ ) {
			if( i > 0 ) {
				classpathStr.append(File.pathSeparatorChar);
			}
			classpathStr.append(classpath[i].toString());
		}
		if( classpath.length > 0 ) {
			options.add("-classpath");
			options.add(classpathStr.toString());
		}		
	}
	
	private void buildSourcepathOption( ProjectModel project, List options ) {
		options.add("-sourcepath");
		options.add(project.getSourceRoot().getAbsolutePath());
	}
	
	private void buildDestinationOptions( ProjectModel project, List options ) {
		options.add("-d");
		options.add(project.getDestinationRoot().getAbsolutePath());
	}
	
	private String[] extractStringArray( List list ) {
		String[] rc = new String[list.size()];
		System.arraycopy( list.toArray(), 0, rc, 0, rc.length );
		return rc;
	}
	
}

/* Old compile file
		ProjectModel project = MainFrame.getMainFrame().getProject();
		if( project != null ) {
        	JDesktopPane desktop = m_mainFrame.getDesktop();
        	JInternalFrame[] frames = desktop.getAllFrames();
    		if( frames.length == 0 ) {
    			return;
    		}
    		File javaFile = ((InternalTextFrame)frames[0]).getFile();
    		String javaFilename = javaFile.getAbsolutePath();

			StringBuffer javaCmd = new StringBuffer();
			ArrayList args = new ArrayList(20);

			javaCmd.append(project.getJDKRoot());
			if( !javaCmd.toString().endsWith(File.separator) ) {
				javaCmd.append(File.separator);
			}
			javaCmd.append("bin");
			javaCmd.append(File.separator);		
			javaCmd.append("javac");
			args.add(javaCmd.toString());

			args.add(javaFilename);

			Object[] objArray = args.toArray();

			for( int i=0; i<objArray.length; i++ ) {
				System.out.print(" "+objArray[i].toString());
			}
			System.out.println();

			try {
				String[] argArray = new String[objArray.length];
				System.arraycopy(objArray, 0, argArray, 0, objArray.length);
				Process proc = Runtime.getRuntime().exec(argArray);
				boolean bContinue = true;
				BufferedInputStream in = new BufferedInputStream(proc.getInputStream());				
				BufferedInputStream err = new BufferedInputStream(proc.getErrorStream());
				while( bContinue ) {
					while( in.available() > 0 ) {
						System.out.print((char)in.read());
					}
					System.out.flush();
					while( err.available() > 0 ) {
						System.err.print((char)err.read());
					}
					System.err.flush();
					try {
						proc.exitValue();
						bContinue = false;
					} catch(  IllegalThreadStateException e ) {
						// do nothing
						bContinue = true;
					}
				}
				while( in.available() > 0 ) {
					System.out.print((char)in.read());
				}
				System.out.flush();
				while( err.available() > 0 ) {
					System.err.print((char)err.read());
				}
				System.err.flush();
				try {
					System.out.println("Compile finished with "+proc.exitValue());
				} catch( IllegalThreadStateException e ) {
					System.err.println(e);
				}
			} catch( IOException e ) {
				System.err.println(e);
			}
		}
*/