/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui;

import javax.swing.*;
import javax.swing.border.*;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;

public class WindowList 
	extends javax.swing.JPanel 
	implements	ActionListener,
				ListSelectionListener,
				MouseListener,
				ContainerListener,
				PropertyChangeListener,
				AncestorListener {
	MainFrame m_mainFrame = null;
		
	public WindowList( MainFrame mainFrame ) {
		m_mainFrame = mainFrame;
		
		//{{INIT_CONTROLS
		setBorder(m_emptyBorder);
		setLayout(new BorderLayout(10,10));
		setSize(430,270);
		m_listPanel.setLayout(new BorderLayout(0,0));
		add(BorderLayout.CENTER,m_listPanel);
		m_selectLabel.setText("Select window:");
		m_listPanel.add(BorderLayout.NORTH,m_selectLabel);
		m_selectLabel.setForeground(java.awt.Color.black);
//		m_selectLabel.setFont(new Font("Dialog", Font.PLAIN, 12));
		m_listPanel.add(BorderLayout.CENTER,m_windowList);
		m_buttonPackPanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
		add(BorderLayout.EAST,m_buttonPackPanel);
		m_buttonPanel.setLayout(new GridLayout(6,1,0,5));
		m_buttonPackPanel.add(m_buttonPanel);
		m_activateButton.setActionCommand("ACTIVATE");
		m_activateButton.setMnemonic((int)'A');
		m_buttonPanel.add(m_activateButton);
//		m_activateButton.setFont(new Font("Dialog", Font.PLAIN, 12));
		m_okButton.setActionCommand("OK");
		m_buttonPanel.add(m_okButton);
//		m_okButton.setFont(new Font("Dialog", Font.PLAIN, 12));
		m_saveButton.setActionCommand("SAVE");
		m_saveButton.setMnemonic((int)'S');
		m_buttonPanel.add(m_saveButton);
//		m_saveButton.setFont(new Font("Dialog", Font.PLAIN, 12));
		m_closeButton.setActionCommand("CLOSE");
		m_closeButton.setMnemonic((int)'C');
		m_buttonPanel.add(m_closeButton);
//		m_closeButton.setFont(new Font("Dialog", Font.PLAIN, 12));
		m_tileHorizontalButton.setActionCommand("TILEHORIZONTAL");
		m_tileHorizontalButton.setMnemonic((int)'H');
		m_buttonPanel.add(m_tileHorizontalButton);
//		m_tileHorizontalButton.setFont(new Font("Dialog", Font.PLAIN, 12));
		m_tileVerticalButton.setActionCommand("TILEVERTICAL");
		m_tileVerticalButton.setMnemonic((int)'V');
		m_buttonPanel.add(m_tileVerticalButton);
//		m_tileVerticalButton.setFont(new Font("Dialog", Font.PLAIN, 12));
		//$$ m_emptyBorder.move(0,271);
		//$$ emptyBorder1.move(0,271);
		//}}

		//{{REGISTER_LISTENERS
		m_activateButton.addActionListener(this);
		m_okButton.addActionListener(this);
		m_saveButton.addActionListener(this);
		m_closeButton.addActionListener(this);
		m_tileHorizontalButton.addActionListener(this);
		m_tileVerticalButton.addActionListener(this);
		
		m_windowList.addListSelectionListener(this);
		m_windowList.addMouseListener(this);
		
		this.addAncestorListener(this);
		//}}
		
    	setWindows( m_mainFrame.getDesktop().getAllFrames() );
		m_windowList.setModel(m_listModel);
		m_windowList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		enableButtons();
	}

	//{{DECLARE_CONTROLS
	javax.swing.JPanel m_listPanel = new javax.swing.JPanel();
	javax.swing.JLabel m_selectLabel = new javax.swing.JLabel();
	javax.swing.JList m_windowList = new javax.swing.JList();
	javax.swing.JPanel m_buttonPackPanel = new javax.swing.JPanel();
	javax.swing.JPanel m_buttonPanel = new javax.swing.JPanel();
	javax.swing.JButton m_activateButton = new javax.swing.JButton("Activate");
	javax.swing.JButton m_okButton = new javax.swing.JButton("OK");
	javax.swing.JButton m_saveButton = new javax.swing.JButton("Save");
	javax.swing.JButton m_closeButton = new javax.swing.JButton("Close Window(s)");
	javax.swing.JButton m_tileHorizontalButton = new javax.swing.JButton("Tile Horizontal");
	javax.swing.JButton m_tileVerticalButton = new javax.swing.JButton("Tile Vertical");
	javax.swing.border.EmptyBorder m_emptyBorder = new javax.swing.border.EmptyBorder(new Insets(10,10,10,10));
	//}}
	
	JInternalFrame[] m_windows = {};
	public void setWindows( JInternalFrame[] windows ) {
		JInternalFrame[] temp = m_windows;
		
		// remove property listeners from the old windows
		for( int i = 0 ; i < temp.length; i++ ) {
			temp[i].removePropertyChangeListener("title", this);
		}
		// add property listeners to the new windows
		for( int i = 0 ; i < windows.length; i++ ) {
			windows[i].addPropertyChangeListener("title", this);
		}
		
		m_windows = windows;
		// Make sure the windows update
		if( temp.length > 0 && m_windows.length > 0 ) {
			m_listModel.fireListDataEvent( new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, Math.min(temp.length, m_windows.length-1)));
		}
		if( temp.length > m_windows.length ) {
			m_listModel.fireListDataEvent( new ListDataEvent(this, ListDataEvent.INTERVAL_REMOVED, m_windows.length, temp.length-1));			
		}
		if( m_windows.length > temp.length ) {
			m_listModel.fireListDataEvent( new ListDataEvent(this, ListDataEvent.INTERVAL_ADDED, temp.length, m_windows.length-1));			
		}		
	}
	
	public void activate() {
		int sel = m_windowList.getSelectedIndex();
		try {
			m_windows[sel].setSelected(true);
			m_windowList.clearSelection();
			m_windowList.setSelectedIndex(0);
		} catch( PropertyVetoException e ) {
			System.err.println(e);
		}
	}
	
	public void ok() {
		// TODO : make this more general. Maybe use a custom listener
		m_windowDialog.setVisible(false);
	}
	
	public void save() {
		JInternalFrame[] windows = m_windows;
		int[] sel = m_windowList.getSelectedIndices();
		for( int i=0; i < sel.length; i++ ) {
    		InternalTextFrame internalFrame = (InternalTextFrame)windows[sel[i]];
    		m_mainFrame.frameSave(internalFrame);
		}		
	}
	
	public void close() {
		JInternalFrame[] windows = m_windows;
		int[] sel = m_windowList.getSelectedIndices();
		for( int i=0; i < sel.length; i++ ) {
    		InternalTextFrame internalFrame = (InternalTextFrame)windows[sel[i]];
    		m_mainFrame.frameClose(internalFrame);
		}		
	}
	
	public void tileHorizontal() {
		JInternalFrame[] windows = m_windows;
		int[] sel = m_windowList.getSelectedIndices();
		if( sel.length > 1 ) {
			// uniconify and tile the rest of the windows
        	JDesktopPane desktop = m_mainFrame.getDesktop();
        	Dimension size = new Dimension(desktop.getSize().width, desktop.getSize().height/sel.length);
        	Point pos = new Point(0,0);
			for( int i = 0; i < sel.length; i++ ) {
    			try {
    				windows[i].setMaximum(false);
    				windows[i].setIcon(false);
    			} catch( PropertyVetoException e ) {
    				System.err.println(e);
    			}
        		windows[i].setSize(size);
        		windows[i].setLocation(pos);
        		pos.y += size.height;
			}
		}
	}
	
	public void tileVertical() {
		JInternalFrame[] windows = m_windows;
		int[] sel = m_windowList.getSelectedIndices();
		if( sel.length > 1 ) {
			// uniconify and tile the rest of the windows
        	JDesktopPane desktop = m_mainFrame.getDesktop();
        	Dimension size = new Dimension(desktop.getSize().width/sel.length, desktop.getSize().height);
        	Point pos = new Point(0,0);
			for( int i = 0; i < sel.length; i++ ) {
    			try {
    				windows[i].setMaximum(false);
    				windows[i].setIcon(false);
    			} catch( PropertyVetoException e ) {
    				System.err.println(e);
    			}
        		windows[i].setSize(size);
        		windows[i].setLocation(pos);
        		pos.x += size.width;
			}
		}
	}
	
	private void enableButtons() {
		int[] sel = m_windowList.getSelectedIndices();
		if( sel.length > 0 ) {
			m_activateButton.setEnabled(true);
			m_saveButton.setEnabled(true);
			m_closeButton.setEnabled(true);
		} else {
			m_activateButton.setEnabled(false);
			m_saveButton.setEnabled(false);
			m_closeButton.setEnabled(false);
		}
		
		if( sel.length > 1 ) {
			m_tileHorizontalButton.setEnabled(true);
			m_tileVerticalButton.setEnabled(true);
			m_activateButton.setEnabled(false);
		} else {
			m_tileHorizontalButton.setEnabled(false);
			m_tileVerticalButton.setEnabled(false);
		}
		
		m_okButton.setEnabled(true);
	}
///////////////////////////////////////////////////////////////////////////////
//	ActionListener
	public void actionPerformed(ActionEvent e) {
		if( e.getActionCommand().equals("ACTIVATE") ) {
			activate();
		} else if( e.getActionCommand().equals("OK") ) {
			ok();
		} else if( e.getActionCommand().equals("SAVE") ) {
			save();
		} else if( e.getActionCommand().equals("CLOSE") ) {
			close();
		} else if( e.getActionCommand().equals("TILEHORIZONTAL") ) {
			tileHorizontal();
		} else if( e.getActionCommand().equals("TILEVERTICAL") ) {
			tileVertical();
		} 
	}
//	ActionListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	ListSelectionListener
	public void valueChanged(ListSelectionEvent e) {
		enableButtons();
	}
//	ListSelectionListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	ListModel
	private DataListModel m_listModel = new DataListModel();
	private class DataListModel implements ListModel {
		EventListenerList m_listModelListener = new EventListenerList();
		public int getSize() {
			return m_windows.length;
		}
		  
		public Object getElementAt(int index) {
			return m_windows[index].getTitle();
		}

		public void addListDataListener(ListDataListener l) {
			m_listModelListener.add(ListDataListener.class, l);
		}

		public void removeListDataListener(ListDataListener l) {
			m_listModelListener.remove(ListDataListener.class, l);
		}
		
		public void fireListDataEvent(ListDataEvent event) {
			// Guaranteed to return a non-null array
			Object[] listeners = m_listModelListener.getListenerList();
			// Process the listeners last to first, notifying
			// those that are interested in this event
			for (int i = listeners.length-2; i>=0; i-=2) {
				if (listeners[i]==ListDataListener.class) {
					switch( event.getType() ) {
						case ListDataEvent.CONTENTS_CHANGED: {
							((ListDataListener)listeners[i+1]).contentsChanged(event);
						} break;
						case ListDataEvent.INTERVAL_ADDED: {
							((ListDataListener)listeners[i+1]).intervalAdded(event);
						} break;
						case ListDataEvent.INTERVAL_REMOVED: {
							((ListDataListener)listeners[i+1]).intervalRemoved(event);
						} break;
					}
				}	       
			}
		}
	};
//	ListModel
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	MouseListener
	public void mouseClicked(MouseEvent event) {
		if( event.getClickCount() == 2 ) {
			int[] sel = m_windowList.getSelectedIndices();
			if( sel.length == 1 ) {
				activate();
				ok();
			}
		}
	}

	public void mouseEntered(MouseEvent e) {}	
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
//	MouseListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	AncestorListener
	public void ancestorAdded(AncestorEvent e) { 
		m_mainFrame.getDesktop().addContainerListener(this);
		setWindows(m_mainFrame.getDesktop().getAllFrames());
	}
	public void ancestorMoved(AncestorEvent e) {}
	public void ancestorRemoved(AncestorEvent e) { 
		m_mainFrame.getDesktop().removeContainerListener(this);
		setWindows( new JInternalFrame[0] );
	}
//	AncestorListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	PropertyChangeListener
	public void propertyChange(PropertyChangeEvent e) { 
		for( int i=0; i < m_windows.length; i++ ) {
			if( m_windows[i] == e.getSource() ) {
				m_listModel.fireListDataEvent(new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, i, i));
			}
		}
	}
//	PropertyChangeListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	ContainerListener
    public void componentAdded(ContainerEvent e) { 
    	setWindows( m_mainFrame.getDesktop().getAllFrames() );
//    	System.err.println(e); 
    }
    
    public void componentRemoved(ContainerEvent e) { 
    	setWindows( m_mainFrame.getDesktop().getAllFrames() );
//    	System.err.println(e); 
    }
//	ContainerListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	KeyListener
//	KeyListener
///////////////////////////////////////////////////////////////////////////////
	static JDialog m_windowDialog = null;
	static WindowList m_windowListPanel = null;
	public static void showDialog( MainFrame mainFrame, JInternalFrame[] windows ) {
		if( m_windowDialog == null ) {
			m_windowDialog = new JDialog(mainFrame, "Window", true);
			m_windowListPanel = new WindowList(mainFrame);
			m_windowDialog.setSize(458, 292);
			m_windowDialog.setResizable(false);
			
			m_windowDialog.getContentPane().setLayout(new BorderLayout());
			m_windowDialog.getContentPane().add(m_windowListPanel, BorderLayout.CENTER);
			
			gvid.Util.recurseAddKeyListener(m_windowDialog, new KeyAdapter() {
				public void keyPressed( KeyEvent evt ) {
					if( evt.getKeyCode() == KeyEvent.VK_ESCAPE ) {
						m_windowListPanel.ok();
					}
				}
			});
		}
		m_windowListPanel.setWindows(windows);
		if( windows.length > 0 ) {
			m_windowListPanel.m_windowList.setSelectedIndex(0);
		} else {
			m_windowListPanel.m_windowList.clearSelection();
		}
		
		Point pos = mainFrame.getLocation();
		Dimension size1 = mainFrame.getSize();
		Dimension size2 = m_windowDialog.getSize();
		m_windowDialog.setLocation( pos.x+size1.width/2-size2.width/2, pos.y+size1.height/2-size2.height/2 );
		
		m_windowListPanel.m_okButton.requestFocus();
		m_windowDialog.setVisible(true);
	}
}