/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.editor;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class AutoscrollController {
	static final int[] SCROLLTIMINGS = {
		100,  56,  23 };
	
	SourceCodeEditor m_editor = null;
	Point m_verticalScrollPoint = new Point();
	Point m_horizontalScrollPoint = new Point();
	Timer m_verticalTimer = new Timer(SCROLLTIMINGS[0], new VerticalScroller());
	Timer m_horizontalTimer = new Timer(250, new HorizontalScroller());
	int m_nVerticalTimeCount = 0;
	int m_nVerticalScrollCount = 0;
	int m_modifiers = 0;
	
	public AutoscrollController( SourceCodeEditor editor ) {
		m_editor = editor;
	}
	
	public void startHorizontalAutoscroll( Point p, int modifiers ) {
		m_modifiers = modifiers;
		m_horizontalScrollPoint.x = p.x;
		m_horizontalScrollPoint.y = p.y;
		m_horizontalTimer.setDelay(250);
		m_horizontalTimer.restart();
	}
	
	public void startVerticalAutoscroll( Point p, int modifiers ) {
		m_modifiers = modifiers;
		m_verticalScrollPoint.x = p.x;
		m_verticalScrollPoint.y = p.y;
		Rectangle b = m_editor.getEditBounds();
		if( m_verticalScrollPoint.y < b.y ) {
			m_nVerticalTimeCount = (b.y - m_verticalScrollPoint.y);
		} else if( m_verticalScrollPoint.y > ( b.y + (m_editor.getLastVisibleRow()-m_editor.getFirstVisibleRow()*m_editor.getFontHeight() ) ) && m_editor.getLineCount() > m_editor.getLastVisibleRow()) {
			m_editor.getEditCommands().windowScrollUp();
			m_nVerticalTimeCount = (m_verticalScrollPoint.y - (b.y + ( b.y + (m_editor.getLastVisibleRow()-m_editor.getFirstVisibleRow()*m_editor.getFontHeight() ) )));
		}
		m_nVerticalScrollCount = 0;		
		m_verticalTimer.setDelay( SCROLLTIMINGS[Math.min(m_nVerticalTimeCount/1000, SCROLLTIMINGS.length-1)] );
		m_verticalTimer.restart();
	}
	
	public void stopHorizontalAutoscroll() {
		m_horizontalTimer.stop();
	}
	
	public void stopVerticalAutoscroll() {
		m_verticalTimer.stop();
	}
	
	private class VerticalScroller implements ActionListener {
		public void actionPerformed( ActionEvent e ) {
//			System.err.println(m_nVerticalTimeCount+" "+m_nVerticalScrollCount);
			Rectangle b = m_editor.getEditBounds();
			if( m_verticalScrollPoint.y < b.y ) {
				for( int i=0; i < (m_nVerticalScrollCount/10)+1; i++ ) {
					m_editor.getEditCommands().windowScrollDown();
				}
			} else if( m_verticalScrollPoint.y > b.y + ( b.y + (m_editor.getLastVisibleRow()-m_editor.getFirstVisibleRow()*m_editor.getFontHeight() ) ) && m_editor.getLineCount() > m_editor.getLastVisibleRow()) {
				for( int i=0; i < (m_nVerticalScrollCount/10)+1; i++ ) {
					m_editor.getEditCommands().windowScrollUp();
				}
        	}
			m_nVerticalTimeCount += m_verticalTimer.getDelay();
			if( m_nVerticalTimeCount/1000 >= SCROLLTIMINGS.length ) {
				m_nVerticalScrollCount++;
				m_verticalTimer.setDelay(SCROLLTIMINGS[SCROLLTIMINGS.length-1]);
			} else {
				m_verticalTimer.setDelay(SCROLLTIMINGS[m_nVerticalTimeCount/1000]);
			}
			m_editor.extendViewSelection(new Point(m_verticalScrollPoint), m_modifiers);
		}
	}
	private class HorizontalScroller implements ActionListener {
		public void actionPerformed( ActionEvent e ) {
			Rectangle b = m_editor.getEditBounds();
			if( m_horizontalScrollPoint.x < b.x ) {
				m_editor.getEditCommands().windowScrollLeft();
			} else if( m_horizontalScrollPoint.x > b.x + (m_editor.getLastVisibleColumn()-m_editor.getFirstVisibleColumn())*m_editor.getFontWidth() ) {
				m_editor.getEditCommands().windowScrollRight();
            }
			m_editor.extendViewSelection(new Point(m_horizontalScrollPoint), m_modifiers);
		}
	}
}