/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.editor;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import gvid.ui.editor.selection.*;
import gvid.ui.editor.undo.*;
import gvid.undo.UndoChain;
import gvid.util.CursorManager;

public class SourceCodeEditBar 
    extends javax.swing.JComponent
    implements  MouseListener,
                MouseMotionListener
{
    private SourceCodeView m_editor = null;

	public SourceCodeEditBar(SourceCodeView parent)	{
	    m_editor = parent;
        buildCursor(this);
	    setCursor( CursorManager.getInstance().getCursor("RIGHT_DEFAULT") );
	    addMouseListener(this);
	    addMouseMotionListener(this);
	}
		
	public void addNotify() {
	    super.addNotify();
	    updateBounds();
	}
	
    public void updateBounds() {
        Dimension d = getPreferredSize();
        Rectangle bounds = new Rectangle(0, 0, d.width, d.height);
        setBounds(bounds);
        repaint();
    }
    
    public void paint(Graphics g) {
        super.paint(g);
        g.setColor(Color.lightGray);
        Rectangle r = new Rectangle(0, 0, getSize().width, getSize().height);
        ditherArea(r, g);
    }
    
    public Dimension getMinimumSize() {
        return getPreferredSize();
    }
    
    public Dimension getMaximumSize() {
        return getPreferredSize();
    }
    
    public Dimension getPreferredSize() {
        Dimension rc = new Dimension(20,0);
        if( m_editor != null ) {
            rc.height = m_editor.getBounds().height;
        }
        return rc;
    }    
    
    public void copySettingsFrom( SourceCodeEditBar bar ) {    	
    }

	private Image m_ditherImage = null;
	private Rectangle m_lastImageSize = null;
	private void ditherArea( Rectangle b, Graphics g ) {
	    if( m_ditherImage == null || !m_lastImageSize.equals(b) ) {
	        m_lastImageSize = new Rectangle(b);
	        m_ditherImage = createImage(b.width, b.height);
	        Graphics g2 = m_ditherImage.getGraphics();
	        g2.setColor(Color.white);
	        g2.fillRect(0, 0, b.width, b.height);
	        g2.setColor(Color.lightGray);
	        for( int y=b.y-b.width-1; y<b.y+b.height+1; y+=2 ) {
	            g2.drawLine(0,y,b.width+1,y+b.width+1);
	        }
	    }
	    
	    g.drawImage(m_ditherImage, b.x, b.y, null);
	}
	
	private static synchronized void buildCursor(Component c) {
		if( CursorManager.getInstance().getCursor("RIGHT_DEFAULT") == null ) {
            MediaTracker tracker = new MediaTracker(c);
            Image rightDefault = Toolkit.getDefaultToolkit().getImage(ClassLoader.getSystemResource("gvid/ui/icons/RIGHT_DEFAULT.gif"));
            tracker.addImage(rightDefault, 0);
            try {
                tracker.waitForID(0);
            } catch (InterruptedException ie) {
                ie.printStackTrace();
            }
            Toolkit.getDefaultToolkit().createCustomCursor(rightDefault, new Point(12,0), "RIGHT_DEFAULT");
		}
	}
	
	private void selectAndScroll( int row ) {
    	if( row > m_editor.getLineCount() ) {
    		// if we are selecting the last line, select to the end of the line, 
    		// not the beginning of the next line
    		m_editor.setSelectionMode( SourceCodeSelection.LINE );
    		m_editor.setSelectionPosition(
    			m_editor.docToPos(
    				new Point(
    					m_editor.getLineLength(m_editor.getLineCount()),
    					m_editor.getLineCount())));
    			
    	} else {
    		m_editor.setSelectionMode(SourceCodeSelection.LINE);
	    	m_editor.setSelectionPosition(new Point(0, row));
    	}
    	m_editor.scrollToCaret();
	}
///////////////////////////////////////////////////////////////////////////////
//  MouseListener
    public void mouseClicked(MouseEvent e) {
    }
    public void mousePressed(MouseEvent e) {
    	if( (e.getModifiers()&InputEvent.BUTTON1_MASK) != 0 ) {
    		SelectionCommand undo = new SelectionCommand("mousePressed");
    		undo.initSelection(m_editor);
    		Point pos = m_editor.viewToPosition(e.getPoint());
    		if( (e.getModifiers()&InputEvent.SHIFT_MASK) == 0 ) {
    			m_editor.setCaretPosition(new Point(0, pos.y), true);
    		}
    		selectAndScroll(pos.y);
    		undo.finalSelection(m_editor);
    	}
    }
    public void mouseReleased(MouseEvent e) {
    	if( (e.getModifiers()&InputEvent.BUTTON1_MASK) != 0 ) {
    		UndoChain undoCmd = new UndoChain("lineSelection");
    		SourceCodeSelection sel = m_editor.getSelection();
    		if( ( sel != null ) && ( sel instanceof LineSelection ) ) {
    			Point front = sel.getSelectionStart();
    			Point back = sel.getSelectionEnd();
    			if( back.y < front.y ) {
    				Point temp = front;
    				front = back;
    				back = temp;
    			}
    			
				if( back.y < m_editor.getFirstLine() ) {
					back.y = m_editor.getFirstLine();
					back.x = 0;
				} else if( back.y > m_editor.getLineCount() ) {
					back.y = m_editor.getLineCount();
					back.x = m_editor.getLineLength(back.y);
				}
    			
    			undoCmd.addRedo(m_editor.setCaretPosition(m_editor.docToPos(front), true));
    			undoCmd.addRedo(m_editor.setSelectionMode(SourceCodeSelection.BOUNDED));
    			undoCmd.addRedo(m_editor.setSelectionPosition(m_editor.docToPos(back)));
    			m_editor.getSourceModel().getUndoManager().addUndo(undoCmd);
    		}
    	}
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
    }
//  MouseListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//  MouseMotionListener
    public void mouseDragged(MouseEvent e) {
    	SelectionCommand undo = new SelectionCommand( "mouseDragged");
    	undo.initSelection(m_editor);
    	Point pos = m_editor.viewToPosition(e.getPoint());
    	selectAndScroll(pos.y);
    	undo.finalSelection(m_editor);
    }
    
    public void mouseMoved(MouseEvent e) {
    }
//  MouseMotionListener
///////////////////////////////////////////////////////////////////////////////
}