/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.editor;

import java.awt.FontMetrics;
import java.awt.Point;
import java.awt.Rectangle;
import gvid.command.EditCommands;
import gvid.ui.editor.selection.SourceCodeSelection;
import gvid.undo.UndoRedoManager;
import gvid.document.SourceCodeModel;
import gvid.document.Line;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import gvid.util.ReferencePointListener;
import gvid.undo.UndoCommand;
import gvid.ui.status.StatusSource;

public interface SourceCodeEditor {
	public UndoRedoManager getUndoManager();
    public int getFontHeight();    
    public int getFontWidth();  
    public FontMetrics getFontMetrics();
    public int getFirstVisibleColumn();
    public int getFirstVisibleRow();
    public void setFirstVisibleRow( int row );
    public void setFirstVisibleColumn( int col );
    public int getLastVisibleColumn();
    public int getLastVisibleRow();
    public int getVisibleRowCount();
    public int getVisibleColumnCount();
    public int getColumnLeftEdge(int col);
    public int getRowTopEdge(int row);
    
    public int getTabWidth();
    public boolean isInsert();
    public boolean isOverwrite();
    
    public Point getCaretPosition();
    public UndoCommand setCaretPosition( Point pos, boolean clearSelection );
    public void scrollToCaret();
    
    /**
     * Moves the cursor to the specified position. If there is currently no selection, the 
     * previous value of the cursor becomes the start of the selection. Otherwise, the 
     * start of the selection remains unchanged.
     * 
     * @param col
     * @param row
     */
    public UndoCommand setSelectionPosition( Point selectionPos );    
    public UndoCommand setSelectionMode( int selectionMode );
    public UndoCommand clearSelection();
    public SourceCodeSelection getSelection();
    public UndoCommand extendViewSelection( Point view, int modifiers );
    
    public SourceCodeModel getSourceModel();
    public int getLineLength( int row );
    public int getLineCount();
    public int getFirstLine();
    
    public Rectangle getEditBounds();
    
    public Point viewToPosition( Point viewCoord );
    public Point posToDoc( Point position );
    public Point docToPos( Point doc );
    public Point positionToView( Point position );
    
    public UndoCommand insertChar( char c );
    public UndoCommand deleteRange( Point startPos, Point endPos );
    
    public DataFlavor getPreferredDataFlavor( DataFlavor[] df );
    public Object getPreferredTransferableObject( Transferable t, DataFlavor flavor );
    public UndoCommand insertTransferableObject( Object data, DataFlavor flavor );
    
    public String getText( Point startPos, Point endPos );
    
    public EditCommands getEditCommands();
    
    public boolean addReferencePoint(Point p, String name, ReferencePointListener l);
    public void removeReferencePoint(String name, ReferencePointListener l);
    public String getUnusedReferenceId();
    public void setReferencePoint(Point p, String name, Object caller);
    
    public void repaint();
    public StatusSource getStatusSource();
}