/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.editor.selection;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import gvid.ui.editor.SourceCodeEditor;
import gvid.document.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.io.*;
import gvid.util.ReferencePointListener;
import gvid.undo.*;
import gvid.ui.editor.undo.*;

public class BlockSelection 
	extends 	JComponent
	implements	SourceCodeSelection,
				ReferencePointListener,
				HierarchyListener,
				Autoscroll {
	SourceCodeEditor m_editor;
	Point m_posStart = new Point();
	Point m_posEnd = new Point();
	private String m_copyData = "";
	private DragSource m_dragSource = null;
	private DragGestureRecognizer m_dragGestureRecognizer = null;
	private String m_startId = "";
	private String m_endId = "";
	
	public BlockSelection( Point posStart, Point posEnd, SourceCodeEditor editor ) {
		m_posStart = new Point(posStart);
		m_posEnd = new Point(posEnd);
		
		m_editor = editor;
		
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		
		m_dragSource = new DragSource();
		m_dragGestureRecognizer = m_dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_COPY_OR_MOVE, this);
		
		m_startId = "BOUNDED_START_"+editor.getUnusedReferenceId();
		m_endId = "BOUNDED_END_"+editor.getUnusedReferenceId();		

		addHierarchyListener(this);
	}
	
	public BlockSelection( Point start, Point end, String str ) {
		m_posStart = new Point(start);
		m_posEnd = new Point(end);
		m_copyData = str;
	}
	
	public boolean isFocusTraversable() {
		return false;
	}
	
    public Component asComponent() {
    	return this;
    }
	
	public SourceCodeSelection createCopy(SourceCodeEditor editor) {
		return new BlockSelection(m_posStart, m_posEnd, editor);
	}	
	
	public int getSelectionType() {
		return BLOCKED;
	}

	public void setSelectionStart(Point selStart) {
    	SourceCodeEditor editor = m_editor;
		if( editor != null ) {
			m_posStart = editor.docToPos(editor.posToDoc(selStart));
		} else {
			m_posStart.x = selStart.x;
			m_posStart.y = selStart.y;
		}
		updateBounds();
		editor.setReferencePoint(editor.posToDoc(m_posStart), m_startId, this);
	}

	public void setSelectionEnd(Point selEnd) {
    	SourceCodeEditor editor = m_editor;
		if( editor != null ) {
//			m_posEnd = editor.docToPos(editor.posToDoc(selEnd));
			m_posEnd.x = selEnd.x;
			m_posEnd.y = selEnd.y;
		} else {
			m_posEnd.x = selEnd.x;
			m_posEnd.y = selEnd.y;
		}
		updateBounds();
		editor.setReferencePoint(editor.posToDoc(m_posEnd), m_endId, this);
	}

	public Point getSelectionEnd() {
		return m_posEnd;
	}
	
	public java.awt.Point getSelectionStart() {
		return m_posStart;
	}
		
	public void paint( Graphics g ) {
    	SourceCodeEditor editor = m_editor;
		Point front = new Point( Math.min(m_posStart.x, m_posEnd.x), Math.min(m_posStart.y, m_posEnd.y) );
		Point back = new Point( Math.max(m_posStart.x, m_posEnd.x), Math.max(m_posStart.y, m_posEnd.y) );
		back.y = Math.min( back.y, editor.getSourceModel().getRowMaximum() );
				
		g.setColor(Color.white);
		g.setXORMode(Color.black);
		
		Point pl = getLocation();
		
		for( int i=front.y; i<=back.y; i++ ) {
			Point viewStart = editor.positionToView(editor.docToPos(editor.posToDoc(new Point(front.x, i))));
			Point docEnd = editor.posToDoc(new Point(back.x, i));
			Point posEnd = editor.docToPos(docEnd);
			if( editor.getSourceModel().lineLength(i) == docEnd.x ) {
				posEnd.x = Math.max(posEnd.x, back.x);
			} else {
				posEnd.x = Math.min(posEnd.x, back.x);
			}
			Point viewEnd = editor.positionToView(posEnd);
			
			g.fillRect(viewStart.x-pl.x, viewStart.y-pl.y, viewEnd.x-viewStart.x, viewEnd.y-viewStart.y+editor.getFontHeight());
		}
		
		g.setPaintMode();
	}
	
    public void updateBounds() {
        Point  p = getPreferredLocation();
        Dimension d = getPreferredSize();
        Rectangle bounds = new Rectangle(p.x, p.y, d.width, d.height);
        setBounds(bounds);
        if( getParent() != null ) {
            getParent().invalidate();
        }
        repaint();
    }
    
	public void swapAnchor() {
    	SourceCodeEditor editor = m_editor;
		Point temp = m_posStart;
		m_posStart = m_posEnd;
		m_posEnd = temp;
		editor.setReferencePoint(editor.posToDoc(m_posStart), m_startId, this);
		editor.setReferencePoint(editor.posToDoc(m_posEnd), m_endId, this);
	}
	
    public Point getPreferredLocation() {
    	SourceCodeEditor editor = m_editor;
		Point front = new Point( Math.min(m_posStart.x, m_posEnd.x), Math.min(m_posStart.y, m_posEnd.y) );
		Point back = new Point( Math.max(m_posStart.x, m_posEnd.x), Math.max(m_posStart.y, m_posEnd.y) );
		
		Point rc = editor.positionToView(editor.docToPos(editor.posToDoc(front)));
		for( int i=front.y; i<=back.y; i++ ) {
			Point viewStart = editor.positionToView(editor.docToPos(editor.posToDoc(new Point(front.x, i))));
			rc.x = Math.min(rc.x, viewStart.x);
		}
		Rectangle editBounds = editor.getEditBounds();
		rc.x = Math.max(rc.x, editBounds.x);
        return rc;
    }

    public Dimension getMinimumSize() {
        return getPreferredSize();
    }
    
    public Dimension getMaximumSize() {
        return getPreferredSize();
    }
    
    public Dimension getPreferredSize() {
    	SourceCodeEditor editor = m_editor;
        Dimension rc = new Dimension();
        
		Rectangle editBounds = editor.getEditBounds();
		Point front = new Point( Math.min(m_posStart.x, m_posEnd.x), Math.min(m_posStart.y, m_posEnd.y) );
		Point back = new Point( Math.max(m_posStart.x, m_posEnd.x), Math.max(m_posStart.y, m_posEnd.y) );
		for( int i=front.y; i<=back.y; i++ ) {
			Point viewStart = editor.positionToView(editor.docToPos(editor.posToDoc(new Point(front.x, i))));
			Point docEnd = editor.posToDoc(new Point(back.x, i));
			Point posEnd = editor.docToPos(docEnd);
			posEnd.x = Math.max(posEnd.x, back.x);
			Point viewEnd = editor.positionToView(posEnd);
			viewStart.x = Math.max( viewStart.x, editBounds.x );
			viewEnd.x = Math.max( viewEnd.x, editBounds.x );
			rc.width = Math.max(rc.width, viewEnd.x-viewStart.x);
		}
		rc.height = (back.y-front.y+1)*editor.getFontHeight();
		
        return rc;
    }
    
    public boolean containsPos( Point pos ) {
    	boolean rc = true;    	
    	if( pos.y < Math.min(m_posStart.y, m_posEnd.y) ) {
    		rc = false;
    	}
    	if( pos.y > Math.max(m_posStart.y, m_posEnd.y) ) {
    		rc = false;
    	}
    	if( pos.x <= Math.min(m_posStart.x, m_posEnd.x) ) {
    		rc = false;
    	}
    	if( pos.x >= Math.max(m_posStart.x, m_posEnd.x) ) {
    		rc = false;
    	}    	    	    	    	
    	return rc;
    }
    
    public boolean contains( Point p ) {
    	return contains(p.x, p.y);
    }
    
    public boolean contains( int x, int y ) {
    	boolean rc = false;
    	SourceCodeEditor editor = m_editor;
		Point front = new Point( Math.min(m_posStart.x, m_posEnd.x), Math.min(m_posStart.y, m_posEnd.y) );
		Point back = new Point( Math.max(m_posStart.x, m_posEnd.x), Math.max(m_posStart.y, m_posEnd.y) );
				
		Point pl = getLocation();
		
		for( int i=front.y; i<=back.y; i++ ) {
			Point viewStart = editor.positionToView(editor.docToPos(editor.posToDoc(new Point(front.x, i))));
			Point docEnd = editor.posToDoc(new Point(back.x, i));
			Point posEnd = editor.docToPos(docEnd);
			if( editor.getSourceModel().lineLength(i) == docEnd.x ) {
				posEnd.x = Math.max(posEnd.x, back.x);
			} else {
				posEnd.x = Math.min(posEnd.x, back.x);
			}
			Point viewEnd = editor.positionToView(posEnd);
			Rectangle r = new Rectangle(viewStart.x-pl.x, viewStart.y-pl.y, viewEnd.x-viewStart.x, viewEnd.y-viewStart.y+editor.getFontHeight());
			if( r.contains(x,y) ) {
				rc = true;
				break;
			}
		}
    	return rc;
    }

    private void setAppropriateCursor(Point p) {
		if( contains(p) ) {
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		} else {
			setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
		}
    }
    
    public String getSelectedText() {
    	SourceCodeEditor editor = m_editor;
    	if( editor == null ) {
    		return m_copyData;
    	} else {
    		Point p1 = new Point(Math.min(m_posStart.x, m_posEnd.x), Math.min(m_posStart.y, m_posEnd.y));
    		Point p2 = new Point(Math.max(m_posStart.x, m_posEnd.x), Math.max(m_posStart.y, m_posEnd.y));
	    	
    		StringBuffer rc = new StringBuffer();
    		for( int i=p1.y; i<=p2.y; i++ ) {
    			Point d1 = editor.posToDoc(new Point(p1.x,i));
    			Point d2 = editor.posToDoc(new Point(p2.x,i));
    			rc.append(editor.getSourceModel().getText(d1, d2));
    			if( i <= p2.y-1 ) {
    				rc.append("\n");
    			}
    		}
	    	return rc.toString();
    	}
    }
    
    private UndoCommand deleteSelection(Point docStart, Point docEnd) {
    	UndoChain rc = new UndoChain("blockSelectionDelete");
    	SourceCodeEditor editor = m_editor;
    	rc.addRedo(editor.clearSelection());
    	for( int i=docStart.y; i<=docEnd.y; i++ ) {
    		Point d1 = new Point(docStart.x,i);
    		Point d2 = new Point(docEnd.x,i);
    		rc.addRedo(editor.getSourceModel().deleteRange(d1, d2));
    	}
    	editor.repaint();
	   	return rc;
    }
    
    public UndoCommand paste( SourceCodeEditor editor, Point dest ) {
    	Point docDest = editor.posToDoc(dest);
		m_copyData = getSelectedText();
    	UndoCommand rc = insertText(editor, m_copyData, docDest);
    	return rc;
    }
    
    private UndoCommand insertText( SourceCodeEditor editor, String str, Point dest ) {
    	UndoChain rc = new UndoChain("BlockSelectionInsertion");
    	Point currentPos = new Point(dest);
    	for( int i=0; i<str.length(); i++ ) {
    		char c = str.charAt(i);
    		if( c == '\n' ) {
    			// new line - insert the new line
//    			editor.getSourceModel().split(currentPos, refPos);
    			currentPos.x = dest.x;
    			currentPos.y++;
    			editor.setCaretPosition(currentPos, false);
    			if( currentPos.y > editor.getLineCount() ) {
    				rc.addRedo(editor.getSourceModel().addLine(new Line("")));
    			}
    		} else if( c == '\r' ) {
    			// ignore - '\n' will handle line feeds only
    		} else {
    			// insert the character into the current line
    			rc.addRedo(editor.getSourceModel().insertChar(c, currentPos));
    			currentPos.x++;
    		}
    	}
    	return rc;
    }
    
    public UndoCommand delete() {
    	SourceCodeEditor editor = m_editor;
    	Point p1 = new Point(Math.min(m_posStart.x, m_posEnd.x), Math.min(m_posStart.y, m_posEnd.y));
    	Point p2 = new Point(Math.max(m_posStart.x, m_posEnd.x), Math.max(m_posStart.y, m_posEnd.y));
    	
		Point docStart = editor.posToDoc(p1);
		Point docEnd = editor.posToDoc(p2);
		
    	UndoCommand rc = deleteSelection(docStart, docEnd);    	
    	return rc;
    }
    
	public SourceCodeSelection copy() {
		BlockSelection rc = null;
    	SourceCodeEditor editor = m_editor;
		if( editor != null ) {
			rc = new BlockSelection(getSelectionStart(), getSelectionEnd(), getSelectedText());
		} else {
			rc = new BlockSelection(getSelectionStart(), getSelectionEnd(), m_copyData);
		}
		return rc;
	}
	
///////////////////////////////////////////////////////////////////////////////
//	ClipboardOwner
	public void lostOwnership(Clipboard clipboard, Transferable contents) {
	}
//	ClipboardOwner
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	Transferable
	public Object getTransferData(DataFlavor flavor) 
		throws UnsupportedFlavorException, IOException {
		Object rc = null;
		if( flavor.getRepresentationClass().isInstance(this)) {
			rc = this;
		} else {
			m_stringSelection = new StringSelection(getSelectedText());
			rc = m_stringSelection.getTransferData(flavor);
		}
		return rc;
	}
	
	private StringSelection m_stringSelection = new StringSelection("");
	private static DataFlavor[] availableFlavors = null;		
	public DataFlavor[] getTransferDataFlavors() {
		if( availableFlavors == null ) {
			DataFlavor[] ss = m_stringSelection.getTransferDataFlavors();
			availableFlavors = new DataFlavor[ss.length+1];
			for( int i=0; i < ss.length; i++ ) {
				availableFlavors[i] = ss[i];
			}
			availableFlavors[ss.length] = new DataFlavor(SourceCodeSelection.class, "Block source code selection");
		}
		return (DataFlavor[])availableFlavors.clone();
	} 
	
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		boolean rc = false;
		DataFlavor[] df = getTransferDataFlavors();
		for( int i=0; i<df.length; i++ ) {
			if( df[i].equals(flavor) ) {
				rc = true;
				break;
			}
		}
		return rc;
	}
//	Transferable	
///////////////////////////////////////////////////////////////////////////////

    public UndoCommand toUppercase() {
    	UndoChain rc = new UndoChain("toUppercase");
    	SourceCodeEditor editor = m_editor;
    	
		Point front = new Point( Math.min(m_posStart.x, m_posEnd.x), Math.min(m_posStart.y, m_posEnd.y) );
		Point back = new Point( Math.max(m_posStart.x, m_posEnd.x), Math.max(m_posStart.y, m_posEnd.y) );
				
		rc.addRedo(editor.clearSelection());		
		for( int row=front.y; row<=back.y; row++ ) {
			Point rowFront = new Point(front.x, row);
			Point rowBack = new Point(back.x, row);
			String newStr = editor.getText(rowFront, rowBack);
			newStr = newStr.toUpperCase();
			rc.addRedo( editor.deleteRange(rowFront, rowBack) );
			rc.addRedo( editor.getSourceModel().insertText(newStr, rowFront) );
		}
		
		rc.addRedo(editor.setCaretPosition(m_posStart, true));
    	rc.addRedo(editor.setSelectionMode(SourceCodeSelection.BLOCKED));
		rc.addRedo(editor.setSelectionPosition(m_posEnd));
    	return rc;
    }
    
    public UndoCommand toLowercase() {
    	UndoChain rc = new UndoChain("toLowercase");
    	SourceCodeEditor editor = m_editor;
    	
		Point front = new Point( Math.min(m_posStart.x, m_posEnd.x), Math.min(m_posStart.y, m_posEnd.y) );
		Point back = new Point( Math.max(m_posStart.x, m_posEnd.x), Math.max(m_posStart.y, m_posEnd.y) );
				
		rc.addRedo(editor.clearSelection());		
		for( int row=front.y; row<=back.y; row++ ) {
			Point rowFront = new Point(front.x, row);
			Point rowBack = new Point(back.x, row);
			String newStr = editor.getText(rowFront, rowBack);
			newStr = newStr.toLowerCase();
			rc.addRedo( editor.deleteRange(rowFront, rowBack) );
			rc.addRedo( editor.getSourceModel().insertText(newStr, rowFront) );
		}
		
		rc.addRedo(editor.setCaretPosition(m_posStart, true));
    	rc.addRedo(editor.setSelectionMode(SourceCodeSelection.BLOCKED));
		rc.addRedo(editor.setSelectionPosition(m_posEnd));
    	return rc;
    }
    
    public UndoCommand toggleCase() {
    	UndoChain rc = new UndoChain("toggleCase");
    	SourceCodeEditor editor = m_editor;
    	
		Point front = new Point( Math.min(m_posStart.x, m_posEnd.x), Math.min(m_posStart.y, m_posEnd.y) );
		Point back = new Point( Math.max(m_posStart.x, m_posEnd.x), Math.max(m_posStart.y, m_posEnd.y) );
				
		rc.addRedo(editor.clearSelection());		
		for( int row=front.y; row<=back.y; row++ ) {
			Point rowFront = new Point(front.x, row);
			Point rowBack = new Point(back.x, row);
			String str = editor.getText(rowFront, rowBack);
			
			StringBuffer newStr = new StringBuffer();
			for( int i=0; i<str.length(); i++ ) {
				if( Character.isLetter(str.charAt(i)) ) {
					if( Character.isUpperCase(str.charAt(i)) ) {
						newStr.append(Character.toLowerCase(str.charAt(i)));
					} else {
						newStr.append(Character.toUpperCase(str.charAt(i)));
					}
				} else {
					newStr.append(str.charAt(i));
				}
			}			
			
			rc.addRedo( editor.deleteRange(rowFront, rowBack) );
			rc.addRedo( editor.getSourceModel().insertText(newStr.toString(), rowFront) );
		}
		
		rc.addRedo(editor.setCaretPosition(m_posStart, true));
    	rc.addRedo(editor.setSelectionMode(SourceCodeSelection.BLOCKED));
		rc.addRedo(editor.setSelectionPosition(m_posEnd));
    	return rc;
    }
    
    private void cleanup() {
    	removeHierarchyListener(this);
    	m_dragGestureRecognizer.removeDragGestureListener(this);
    	m_dragGestureRecognizer = null;
    	m_dragSource = null;
    	m_editor = null;
    }
    
///////////////////////////////////////////////////////////////////////////////
//	DragGestureListener
    public void dragGestureRecognized(DragGestureEvent dge) {
		m_stringSelection = new StringSelection(getSelectedText());
    	if( (dge.getDragAction()&DnDConstants.ACTION_COPY) != 0 ) {
        	DragSource.getDefaultDragSource().startDrag(dge, DragSource.DefaultCopyDrop, this, this);
    	} else if( (dge.getDragAction()&DnDConstants.ACTION_MOVE) != 0 ) {
	        DragSource.getDefaultDragSource().startDrag(dge, DragSource.DefaultMoveDrop, this, this);
    	}    	
    }
//	DragGestureListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	DragSourceListener
    public void dragEnter(DragSourceDragEvent dsde) {
    }
    public void dragOver(DragSourceDragEvent dsde) {
    }
    public void dropActionChanged(DragSourceDragEvent dsde) {
    }
    public void dragExit(DragSourceEvent dse) {
    }
    public void dragDropEnd(DragSourceDropEvent dsde) {
    	if( dsde.getDropSuccess() ){
			DragAndDropCommand undoCmd = new DragAndDropCommand("dragAndDrop", this);
    		SourceCodeEditor editor = m_editor;
        	SelectionCommand selCmd = new SelectionCommand("dragAndDrop");
        	undoCmd.addRedo(selCmd);
        	selCmd.initSelection(editor);
    		m_copyData = getSelectedText();
			// creating this undo command with this allows the drag and drop commands to be combined if necessary.
			if( dsde.getDropAction()==DnDConstants.ACTION_MOVE ) {
				UndoCommand delCmd = delete();
        		undoCmd.addRedo(delCmd);
        	}
        	if( m_editor != null ) {
        		editor.clearSelection();
        	}
        	selCmd.finalSelection(editor);
        	editor.getUndoManager().addUndo(undoCmd);
    	}
    }
//	DragSourceListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	ReferencePointListener
	public void pointChanged( Point p, String name ) {
    	SourceCodeEditor editor = m_editor;
		if( name.equals(m_startId) ) {
			m_posStart = editor.docToPos(p);
			updateBounds();
		} else if( name.equals(m_endId ) ) {
			m_posEnd = editor.docToPos(p);
			updateBounds();
		}
	}
	
	public String changeReferenceName( String name ) {
    	SourceCodeEditor editor = m_editor;
		if( name.equals(m_startId) ) {
			m_startId = "BLOCK_START_"+editor.getUnusedReferenceId();
			return m_startId;
		} else if( name.equals(m_endId ) ) {
			m_endId = "BLOCK_END_"+editor.getUnusedReferenceId();
			return m_endId;
		}
		return "";
	}
//	ReferencePointListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	HierarchyListener
	public void hierarchyChanged(HierarchyEvent e) {
    	SourceCodeEditor editor = m_editor;
		if( (e.getChangeFlags()&HierarchyEvent.PARENT_CHANGED) != 0 ) {
			if( e.getChanged() == this ) {
				if( getParent() != null ) {
	    			updateBounds();
					editor.addReferencePoint(editor.posToDoc(m_posStart), m_startId, this);
					editor.addReferencePoint(editor.posToDoc(m_posEnd), m_endId, this);
				} else {
					editor.removeReferencePoint(m_startId, this);
					editor.removeReferencePoint(m_endId, this);
					cleanup();
				}
			}
		}
	}
//	HierarchyListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	Autoscroll
	public Insets getAutoscrollInsets() {
		return new Insets(getHeight()/2, getWidth()/2, getHeight()/2, getWidth()/2);
	}
	
	public void autoscroll(Point cursorLocn) {
		if( ( getParent() != null ) && ( getParent() instanceof Autoscroll ) ) {
			cursorLocn = SwingUtilities.convertPoint(this, cursorLocn, getParent());
			Rectangle b = getParent().getBounds();
			Insets asi = ((Autoscroll)getParent()).getAutoscrollInsets();
			if( ( cursorLocn.y < asi.top ) || 
				( cursorLocn.y > b.height-asi.bottom ) ||
				( cursorLocn.x < asi.left ) ||
				( cursorLocn.x > b.width-asi.right ) ) {
//				System.err.println("Autoscrolling parent");
				((Autoscroll)getParent()).autoscroll(cursorLocn);
			}
		}
	}
//	Autoscroll
///////////////////////////////////////////////////////////////////////////////
}