/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.editor.undo;

import gvid.undo.*;
import java.awt.Point;
import gvid.document.*;
import gvid.ui.editor.SourceCodeEditor;

public class InsertCharCommand 
	extends InsertDeleteCharBase {
		
	InsertCharCommand m_undoChain = null;
	
	public InsertCharCommand(String name, SourceCodeModel model, char c, Point doc ) {
		super( name, model, c, doc );
	}
	
	public void undo(SourceCodeEditor currentEditor) {
		if( m_undoChain != null ) {
			m_undoChain.undo(currentEditor);
		}
		deleteChar();
		currentEditor.setCaretPosition(currentEditor.docToPos(m_doc), true);
	}
	
	public void redo(SourceCodeEditor currentEditor) {
		insertChar();
		if( m_undoChain != null ) {
			m_undoChain.redo(currentEditor);
		}
	}
	
    public boolean combineCommands( UndoCommand cmd ) {
    	boolean rc = false;
		if( cmd instanceof UndoChain ) {
    		UndoChain chain = (UndoChain)cmd;
    		InsertCharCommand insertCmd = null;
    		for( int i = 0; i < chain.commandList().size(); i++ ) {
    			UndoCommand chainCmd = (UndoCommand)chain.commandList().get(i);
    			if( chainCmd.isEditCmd() ) {
    				if( chainCmd instanceof InsertCharCommand ) {
    					insertCmd = (InsertCharCommand)chainCmd;
    				} else {
    					insertCmd = null;
    					break;
    				}
    			}
    		}
    		if( insertCmd != null ) {
    			cmd = insertCmd;
    		}
    	}
    	if( cmd instanceof InsertCharCommand ) {
    		InsertCharCommand charCmd = (InsertCharCommand)cmd;
    		if( m_undoChain != null ) {
    			rc = m_undoChain.combineCommands(charCmd);
    		} else if( !Character.isWhitespace(m_char) || Character.isWhitespace(charCmd.m_char) ) {
    			rc = true;
    			m_undoChain = charCmd;
    		}
    	} 
    	return rc;
    }
}