/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.editor.undo;

import java.awt.Point;
import gvid.undo.*;
import gvid.ui.editor.SourceCodeEditor;

public class MoveCaretCommand implements UndoCommand
{
    private String m_name;
    private Point m_new = new Point();
    private Point m_old = new Point();
    private UndoCommand m_cmd = null;
    
    public MoveCaretCommand( int newX, int newY, int oldX, int oldY, String name ) {
        m_name = name;
        m_new.x = newX;
        m_new.y = newY;
        m_old.x = oldX;
        m_old.y = oldY;
    }
    
    public String getName() {
        return m_name;
    }
    
    public void setName( String name ) {
    	m_name = name;
    }

    
    public boolean isEditCmd()
    {
        return false;
    }

    public void redo(SourceCodeEditor currentEditor)
    {
        currentEditor.setCaretPosition(m_new, true);
    	if( m_cmd != null ) {
    		m_cmd.redo(currentEditor);
    	}
    }

    public void undo(SourceCodeEditor currentEditor)
    {
    	if( m_cmd != null ) {
    		m_cmd.undo(currentEditor);
    	}
        currentEditor.setCaretPosition(m_old, true);
    }

    public boolean combineCommands( UndoCommand cmd ) {
    	boolean rc = false;
    	if( cmd instanceof MoveCaretCommand ) {
    		MoveCaretCommand moveCmd = (MoveCaretCommand)cmd;
    		if( moveCmd.m_new.equals(m_new) && moveCmd.m_old.equals(m_old) ) {
    			rc = true;
    		}
    	}
    	return rc;
    }
}