/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.editor.undo;

import gvid.undo.*;
import gvid.ui.editor.*;
import gvid.ui.editor.selection.*;
import java.awt.Point;

public class SelectionCommand implements gvid.undo.UndoCommand
{
    private String m_name;
    private Point m_newStart = new Point();
    private Point m_newEnd = new Point();
    private int m_newType = SourceCodeSelection.BOUNDED;
    private Point m_oldStart = new Point();
    private Point m_oldEnd = new Point();
    private int m_oldType = SourceCodeSelection.BOUNDED;
    
    public SelectionCommand( 
    	String name,
    	Point newStart, Point newEnd, int newType, 
    	Point oldStart, Point oldEnd, int oldType ) {
    	this( name);
    	m_newStart.x = newStart.x;
    	m_newStart.y = newStart.y;
    	m_newEnd.x = newEnd.x;
    	m_newEnd.y = newEnd.y;
    	m_newType = newType;
    	m_oldStart.x = oldStart.x;
    	m_oldStart.y = oldStart.y;
    	m_oldEnd.x = oldEnd.x;
    	m_oldEnd.y = oldEnd.y;
    	m_oldType = oldType;
    }
    
    	
    public SelectionCommand( String name ) {
    	m_name = name;
    }
    
    public void initSelection(SourceCodeEditor currentEditor) {
    	if( currentEditor.getSelection() != null ) {
    		m_oldEnd = currentEditor.getSelection().getSelectionEnd();
    		m_oldStart = currentEditor.getSelection().getSelectionStart();
    		m_oldType = currentEditor.getSelection().getSelectionType();
    	} else {
    		m_oldEnd = currentEditor.getCaretPosition();
    		m_oldStart = new Point(m_oldEnd);
    	}
    }
    
    public void finalSelection(SourceCodeEditor currentEditor) {
    	if( currentEditor.getSelection() != null ) {
    		m_newEnd = currentEditor.getSelection().getSelectionEnd();
    		m_newStart = currentEditor.getSelection().getSelectionStart();
    		m_newType = currentEditor.getSelection().getSelectionType();
    	} else {
    		m_newEnd = currentEditor.getCaretPosition();
    		m_newStart = new Point(m_newEnd);
    	}
    }
    
    public String getName() {
        return m_name;
    }
    
    public void setName( String name ) {
    	m_name = name;
    }

    public boolean isEditCmd() {
        return false;
    }

    public void redo(SourceCodeEditor currentEditor) {
    	currentEditor.setCaretPosition(m_newStart, true);
    	currentEditor.setSelectionMode(m_newType);
    	currentEditor.setSelectionPosition(m_newEnd);
    }

    public void undo(SourceCodeEditor currentEditor) {
    	currentEditor.setCaretPosition(m_oldStart, true);
    	currentEditor.setSelectionMode(m_oldType);
    	currentEditor.setSelectionPosition(m_oldEnd);
    }

    public boolean combineCommands( UndoCommand cmd ) {
    	boolean rc = false;
    	if( cmd instanceof SelectionCommand ) {
    		SelectionCommand selCmd = (SelectionCommand)cmd;
    		if( selCmd.m_newStart.equals(m_newStart) ) {
    			m_newEnd = selCmd.m_newEnd;
    			m_newType = selCmd.m_newType;
    			rc = true;
    		}
    	} else if( cmd instanceof UndoChain ) {
    		// if we have a single command inside of an UndoChain
    		// and the command is a selection command, then combine with this one
    		UndoChain chain = (UndoChain)cmd;
    		if( chain.commandList().size() == 1 ) {
    			UndoCommand chainCmd = (UndoCommand)chain.commandList().get(0);
    			if( chainCmd instanceof SelectionCommand ) {
    				SelectionCommand selCmd = (SelectionCommand)chainCmd;
    				rc = combineCommands(selCmd);
    			}
    		}
    	}
    	return rc;
    }
}