/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.splitscrollpane;

import javax.swing.*;
import javax.swing.border.*;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class SplitScrollPane 
    extends     javax.swing.JComponent
    implements  SplitScrollPaneConstants,
                ActionListener,
                AdjustmentListener,
                FocusListener {
	public SplitScrollPane() {
	    m_mainView = new SplitScrollViewBase();
	    m_mainView.asComponent().addFocusListener(this);
	    
	    setBorder( new gvid.ui.border.NarrowBevelBorder() );
	    setLayout( m_splitScrollLayout );
	    add( HSPLIT, m_hSplit );
	    add( VSPLIT, m_vSplit );
        m_mainVerticalScrollBar.addAdjustmentListener(this);
        m_mainHorizontalScrollBar.addAdjustmentListener(this);
	    add( MAIN_VSCROLLBAR, m_mainVerticalScrollBar);
	    add( MAIN_HSCROLLBAR, m_mainHorizontalScrollBar);
	    m_mainVerticalScrollBar.setModel(m_mainView.getVerticalScrollModel());
	    m_mainHorizontalScrollBar.setModel(m_mainView.getHorizontalScrollModel());
	    add( MAIN_VIEW, m_mainView.asComponent() );
	    m_mainView.setActive(true);
	    
	    m_hSplit.addActionListener(this);
	    m_vSplit.addActionListener(this);
	    addFocusListener(this);
	}
	
	Vector m_splitListeners = new Vector();
	
	public void addSplitListener( SplitScrollListener l ) {
		m_splitListeners.addElement(l);
	}
	
	public void removeSplitListener( SplitScrollListener l ) {
		m_splitListeners.removeElement(l);
	}
	
	private void notifyActivePaneChanged() {
		Enumeration enum = m_splitListeners.elements();
		while( enum.hasMoreElements() ) {
			SplitScrollListener l = (SplitScrollListener)enum.nextElement();
			l.activePaneChanged();
		}
	}
	
	public void setView( SplitScrollView view ) {
		if( m_mainView != null )		{ m_mainView.asComponent().removeFocusListener(this);}
		if( m_northView != null )		{ m_northView.asComponent().removeFocusListener(this);}
		if( m_southView != null )		{ m_southView.asComponent().removeFocusListener(this);}
		if( m_eastView != null )		{ m_eastView.asComponent().removeFocusListener(this);}
		if( m_westView != null )		{ m_westView.asComponent().removeFocusListener(this);}
		if( m_northEastView != null )	{ m_northEastView.asComponent().removeFocusListener(this);}
		if( m_northWestView != null )	{ m_northWestView.asComponent().removeFocusListener(this);}
		if( m_southEastView != null )	{ m_southEastView.asComponent().removeFocusListener(this);}
		if( m_southWestView != null )	{ m_southWestView.asComponent().removeFocusListener(this);}
		
	    view.asComponent().addFocusListener(this);
		if( m_mainView != null ) {
	    	m_mainView = view;
		} else if( m_northView != null ) {
	    	m_northView = view;
	    	m_southView = copyView( view, false, true );
		} else if( m_eastView != null ) {
	    	m_eastView = view;
	    	m_westView = copyView( view, true, false );
		} else if( m_northEastView != null ) {
	    	m_northEastView = view;
	    	m_northWestView = copyView( view, true, true );
	    	m_southEastView = copyView( view, true, true );
	    	m_southWestView = copyView( view, true, true );
		} else {
			// do main view by default
	    	m_mainView = view;
		}
	    
	    // remove old view and add new views    
	    if( m_splitScrollLayout.getLayoutComponent(NW_VIEW) != null ) {
	    	replaceView(NW_VIEW, m_northWestView);
	        m_northVerticalScrollBar.setModel(m_northWestView.getVerticalScrollModel());
	        m_westHorizontalScrollBar.setModel(m_northWestView.getHorizontalScrollModel());
	    } else if( m_splitScrollLayout.getLayoutComponent(NE_VIEW) != null ) {
	    	replaceView(NE_VIEW, m_northEastView);
	        m_northVerticalScrollBar.setModel(m_northEastView.getVerticalScrollModel());
	        m_eastHorizontalScrollBar.setModel(m_northEastView.getHorizontalScrollModel());
	    } else if( m_splitScrollLayout.getLayoutComponent(SW_VIEW) != null ) {
	    	replaceView(SW_VIEW, m_southWestView);
	        m_southVerticalScrollBar.setModel(m_southWestView.getVerticalScrollModel());
	        m_westHorizontalScrollBar.setModel(m_southWestView.getHorizontalScrollModel());
	    } else if( m_splitScrollLayout.getLayoutComponent(SE_VIEW) != null ) {
	    	replaceView(SE_VIEW, m_southEastView);
	        m_southVerticalScrollBar.setModel(m_southEastView.getVerticalScrollModel());
	        m_eastHorizontalScrollBar.setModel(m_southEastView.getHorizontalScrollModel());
	    } else if( m_splitScrollLayout.getLayoutComponent(N_VIEW) != null ) {
	    	replaceView(N_VIEW, m_northView);
	        m_northVerticalScrollBar.setModel(m_northView.getVerticalScrollModel());
	        m_mainHorizontalScrollBar.setModel(m_northView.getHorizontalScrollModel());
	    } else if( m_splitScrollLayout.getLayoutComponent(S_VIEW) != null ) {
	    	replaceView(S_VIEW, m_southView);
	        m_southVerticalScrollBar.setModel(m_southView.getVerticalScrollModel());
	        m_mainHorizontalScrollBar.setModel(m_southView.getHorizontalScrollModel());
	    } else if( m_splitScrollLayout.getLayoutComponent(E_VIEW) != null ) {
	    	replaceView(E_VIEW, m_eastView);
	        m_mainVerticalScrollBar.setModel(m_eastView.getVerticalScrollModel());
	        m_eastHorizontalScrollBar.setModel(m_eastView.getHorizontalScrollModel());
	    } else if( m_splitScrollLayout.getLayoutComponent(W_VIEW) != null ) {
	    	replaceView(W_VIEW, m_westView);
	        m_mainVerticalScrollBar.setModel(m_westView.getVerticalScrollModel());
	        m_westHorizontalScrollBar.setModel(m_westView.getHorizontalScrollModel());
	    } else if( m_splitScrollLayout.getLayoutComponent(MAIN_VIEW) != null ) {
	    	replaceView(MAIN_VIEW, m_mainView);
	        m_mainVerticalScrollBar.setModel(m_mainView.getVerticalScrollModel());
	        m_mainHorizontalScrollBar.setModel(m_mainView.getHorizontalScrollModel());
	    }
	}
	
	private SplitScrollView copyView( SplitScrollView src, boolean shareVert, boolean shareHoriz ) {
		SplitScrollView rc = src.createCopy();
	    rc.asComponent().addFocusListener(this);
		if( shareVert ) {
	    	rc.shareVerticalScrollModel(src);
		}
		if( shareHoriz ) {
	    	rc.shareHorizontalScrollModel(src);
		}
		rc.copySettingsFrom(src);
		return rc;
	}
	
	private void destroyView( SplitScrollView src ) {
		src.asComponent().removeFocusListener(this);
		src.setModel(null);
	}
	
	private void replaceView( String field, SplitScrollView newView ) {
		SplitScrollView oldView = (SplitScrollView)m_splitScrollLayout.getLayoutComponent(field);
		if( oldView != null && oldView.isActive() ) {
			oldView.setActive(false);
			newView.setActive(true);
			remove(oldView.asComponent());
			add(field, newView.asComponent());
		}
	}
	
	private void giveActiveFocus() {
		if( m_mainView != null && m_mainView.isActive() ) {
			(m_mainView.asComponent()).requestFocus();
		} else if( m_northView != null && m_northView.isActive() ) {
			(m_northView.asComponent()).requestFocus();
		} else if( m_southView != null && m_southView.isActive() ) {
			(m_southView.asComponent()).requestFocus();
		} else if( m_eastView != null && m_eastView.isActive() ) {
			(m_eastView.asComponent()).requestFocus();
		} else if( m_westView != null && m_westView.isActive() ) {
			(m_westView.asComponent()).requestFocus();
		} else if( m_northEastView != null && m_northEastView.isActive() ) {
			(m_northEastView.asComponent()).requestFocus();
		} else if( m_northWestView != null && m_northWestView.isActive() ) {
			(m_northWestView.asComponent()).requestFocus();
		} else if( m_southEastView != null && m_southEastView.isActive() ) {
			(m_southEastView.asComponent()).requestFocus();
		} else if( m_southWestView != null && m_southWestView.isActive() ) {
			(m_southWestView.asComponent()).requestFocus();
		}
	}
	
	public SplitScrollView getActiveView() {
		SplitScrollView rc = null;
		if( m_mainView != null && m_mainView.isActive() ) {
			rc = m_mainView;
		} else if( m_northView != null && m_northView.isActive() ) {
			rc = m_northView;
		} else if( m_southView != null && m_southView.isActive() ) {
			rc = m_southView;
		} else if( m_eastView != null && m_eastView.isActive() ) {
			rc = m_eastView;
		} else if( m_westView != null && m_westView.isActive() ) {
			rc = m_westView;
		} else if( m_northEastView != null && m_northEastView.isActive() ) {
			rc = m_northEastView;
		} else if( m_northWestView != null && m_northWestView.isActive() ) {
			rc = m_northWestView;
		} else if( m_southEastView != null && m_southEastView.isActive() ) {
			rc = m_southEastView;
		} else if( m_southWestView != null && m_southWestView.isActive() ) {
			rc = m_southWestView;
		}
		return rc;
	}
	
	public void setViewModel(SplitScrollModel model) {
		if( m_mainView != null )		{ m_mainView.setModel(model);}
		if( m_northView != null )		{ m_northView.setModel(model);}
		if( m_southView != null )		{ m_southView.setModel(model);}
		if( m_eastView != null )		{ m_eastView.setModel(model);}
		if( m_westView != null )		{ m_westView.setModel(model);}
		if( m_northEastView != null )	{ m_northEastView.setModel(model);}
		if( m_northWestView != null )	{ m_northWestView.setModel(model);}
		if( m_southEastView != null )	{ m_southEastView.setModel(model);}
		if( m_southWestView != null )	{ m_southWestView.setModel(model);}
	}
	
	protected SplitScrollPaneLayout m_splitScrollLayout = new SplitScrollPaneLayout(this);
	
    protected SplitScrollView m_northWestView = null;
    protected SplitScrollView m_southWestView = null;
    protected SplitScrollView m_northEastView = null;
    protected SplitScrollView m_southEastView = null;
    
    protected SplitScrollView m_southView = null;
    protected SplitScrollView m_eastView = null;
    protected SplitScrollView m_northView = null;
    protected SplitScrollView m_westView = null;
    
    protected SplitScrollView m_mainView = null;
    
    protected JScrollBar m_northVerticalScrollBar = new JScrollBar(SwingConstants.VERTICAL);
    protected JScrollBar m_southVerticalScrollBar = new JScrollBar(SwingConstants.VERTICAL);
    protected JScrollBar m_mainVerticalScrollBar = new JScrollBar(SwingConstants.VERTICAL);
    
    protected JScrollBar m_eastHorizontalScrollBar = new JScrollBar(SwingConstants.HORIZONTAL);
    protected JScrollBar m_westHorizontalScrollBar = new JScrollBar(SwingConstants.HORIZONTAL);
    protected JScrollBar m_mainHorizontalScrollBar = new JScrollBar(SwingConstants.HORIZONTAL);
    
    protected SplitControl m_hSplit = new SplitControl(SwingConstants.HORIZONTAL);
    protected SplitControl m_vSplit = new SplitControl(SwingConstants.VERTICAL);
    
    protected SplitScrollPane m_scrollpane = null;
    
    private void splitHorizontal() {
        // swap views
        if( m_splitScrollLayout.getLayoutComponent(MAIN_VIEW) != null ) {
            remove(m_mainView.asComponent());
            m_eastView = m_mainView;
            m_westView = copyView(m_mainView, true, false );
	        m_eastHorizontalScrollBar.setModel(m_eastView.getHorizontalScrollModel());
	        m_westHorizontalScrollBar.setModel(m_westView.getHorizontalScrollModel());
            add(E_VIEW, m_eastView.asComponent());
            add(W_VIEW, m_westView.asComponent());
            m_westView.setActive(true);
            m_mainView = null;
        } else {
            remove(m_northView.asComponent());
            remove(m_southView.asComponent());
            m_northWestView = m_northView;
            m_northEastView = copyView(m_northView, true, false );
            m_southWestView = m_southView;
            m_southEastView = copyView(m_southView, true, false );
	        m_eastHorizontalScrollBar.setModel(m_northEastView.getHorizontalScrollModel());
	        m_westHorizontalScrollBar.setModel(m_northWestView.getHorizontalScrollModel());
            add(NW_VIEW, m_northWestView.asComponent());
            add(NE_VIEW, m_northEastView.asComponent());
            add(SW_VIEW, m_southWestView.asComponent());
            add(SE_VIEW, m_southEastView.asComponent());
            if( m_northView.isActive() ) {
            	m_northWestView.setActive(true);
            } else {
            	m_southWestView.setActive(true);
            }
            m_northView = null;
            m_southView = null;
        }
        
        // swap scrollbars
        // remove the main scrollbar
        remove(m_mainHorizontalScrollBar);
        m_mainHorizontalScrollBar.removeAdjustmentListener(this);
        // add the split scrollbar
        m_eastHorizontalScrollBar.addAdjustmentListener(this);
        m_westHorizontalScrollBar.addAdjustmentListener(this);
        add(E_HSCROLLBAR, m_eastHorizontalScrollBar);
        add(W_HSCROLLBAR, m_westHorizontalScrollBar);
    }
    
    private void splitVertical() {
        // swap views
        if( m_splitScrollLayout.getLayoutComponent(MAIN_VIEW) != null ) {
            remove(m_mainView.asComponent());
            m_northView = m_mainView;
            m_southView = copyView(m_mainView, false, true );
	        m_northVerticalScrollBar.setModel(m_northView.getVerticalScrollModel());
	        m_southVerticalScrollBar.setModel(m_southView.getVerticalScrollModel());
            add(N_VIEW, m_northView.asComponent());
            add(S_VIEW, m_southView.asComponent());
            m_northView.setActive(true);
            m_mainView = null;
        } else {
            remove(m_eastView.asComponent());
            remove(m_westView.asComponent());
            m_northWestView = m_westView;
            m_southWestView = copyView(m_westView, false, true );
            m_northEastView = m_eastView;
            m_southEastView = copyView(m_eastView, false, true );
	        m_northVerticalScrollBar.setModel(m_northWestView.getVerticalScrollModel());
	        m_southVerticalScrollBar.setModel(m_southWestView.getVerticalScrollModel());
            add(NW_VIEW, m_northWestView.asComponent());
            add(SW_VIEW, m_southWestView.asComponent());
            add(NE_VIEW, m_northEastView.asComponent());
            add(SE_VIEW, m_southEastView.asComponent());
            if( m_westView.isActive() ) {
            	m_northWestView.setActive(true);
            } else {
            	m_northEastView.setActive(true);
            }
            m_westView = null;
            m_eastView = null;
        }
                        
        // remove the main scrollbar
        remove(m_mainVerticalScrollBar);
        m_mainVerticalScrollBar.removeAdjustmentListener(this);
        // add the split scrollbars
        m_northVerticalScrollBar.addAdjustmentListener(this);
        m_southVerticalScrollBar.addAdjustmentListener(this);
        add(N_VSCROLLBAR, m_northVerticalScrollBar);
        add(S_VSCROLLBAR, m_southVerticalScrollBar);
    }
    
    private void combineHorizontal() {
        // swap views
        if( m_splitScrollLayout.getLayoutComponent(E_VIEW) != null ) {
            remove(m_eastView.asComponent());
            remove(m_westView.asComponent());
            if( m_hSplit.getPosition() == m_hSplit.getInactivePosition() ) {
                m_mainView = m_eastView;
                if( m_westView.isActive() ) {
                	m_westView.setActive(false);
                	m_mainView.setActive(true);
                }
                destroyView(m_westView);
            } else {
                m_mainView = m_westView;
                if( m_eastView.isActive() ) {
                	m_eastView.setActive(false);
                	m_mainView.setActive(true);
                }
                destroyView(m_eastView);
            }
	        m_mainHorizontalScrollBar.setModel(m_mainView.getHorizontalScrollModel());
            add(MAIN_VIEW, m_mainView.asComponent());
            m_eastView = null;
            m_westView = null;
        } else {
            remove(m_northWestView.asComponent());
            remove(m_southWestView.asComponent());
            remove(m_northEastView.asComponent());
            remove(m_southEastView.asComponent());
            if( m_hSplit.getPosition() == m_hSplit.getInactivePosition() ) {
                m_northView = m_northEastView;
                m_southView = m_southEastView;
                if( m_northWestView.isActive() ) {
                	m_northWestView.setActive(false);
                	m_northView.setActive(true);
                }
                if( m_southWestView.isActive() ) {
                	m_southWestView.setActive(false);
                	m_southView.setActive(true);
                }
                destroyView(m_northWestView);
                destroyView(m_southWestView);
            } else {
                m_northView = m_northWestView;
                m_southView = m_southWestView;
                if( m_northEastView.isActive() ) {
                	m_northEastView.setActive(false);
                	m_northView.setActive(true);
                }
                if( m_southEastView.isActive() ) {
                	m_southEastView.setActive(false);
                	m_southView.setActive(true);
                }
                destroyView(m_northEastView);
                destroyView(m_southEastView);
            }
	        m_mainHorizontalScrollBar.setModel(m_southView.getHorizontalScrollModel());
            add(N_VIEW, m_northView.asComponent());
            add(S_VIEW, m_southView.asComponent());
            m_northWestView = null;
            m_southWestView = null;
            m_northEastView = null;
            m_southEastView = null;
        }
                            
        // swap scrollbars
        // remove the split scrollbars
        remove(m_eastHorizontalScrollBar);
        remove(m_westHorizontalScrollBar);
        m_eastHorizontalScrollBar.removeAdjustmentListener(this);
        m_westHorizontalScrollBar.removeAdjustmentListener(this);
        // add the main scrollbar
        m_mainHorizontalScrollBar.addAdjustmentListener(this);
        add(MAIN_HSCROLLBAR, m_mainHorizontalScrollBar);
    }
    
    private void combineVertical() {
        // swap views
        if( m_splitScrollLayout.getLayoutComponent(N_VIEW) != null ) {
            remove(m_northView.asComponent());
            remove(m_southView.asComponent());
            if( m_vSplit.getPosition() == m_vSplit.getInactivePosition() ) {
                m_mainView = m_southView;
                if( m_northView.isActive() ) {
                	m_northView.setActive(false);
                	m_mainView.setActive(true);
                }
                destroyView(m_northView);
            } else {
                m_mainView = m_northView;
                if( m_southView.isActive() ) {
                	m_southView.setActive(false);
                	m_mainView.setActive(true);
                }
                destroyView(m_southView);
            }
	        m_mainVerticalScrollBar.setModel(m_mainView.getVerticalScrollModel());
            add(MAIN_VIEW, m_mainView.asComponent());
            m_northView = null;
            m_southView = null;
        } else {
            remove(m_northWestView.asComponent());
            remove(m_southWestView.asComponent());
            remove(m_northEastView.asComponent());
            remove(m_southEastView.asComponent());
            if( m_vSplit.getPosition() == m_vSplit.getInactivePosition() ) {
                m_eastView = m_southEastView;
                m_westView = m_southWestView;
                if( m_northEastView.isActive() ) {
                	m_northEastView.setActive(false);
                	m_eastView.setActive(true);
                }
                if( m_northWestView.isActive() ) {
                	m_northWestView.setActive(false);
                	m_westView.setActive(true);
                }
                destroyView(m_northEastView);
                destroyView(m_northWestView);
            } else {
                m_eastView = m_northEastView;
                m_westView = m_northWestView;
                if( m_southEastView.isActive() ) {
                	m_southEastView.setActive(false);
                	m_eastView.setActive(true);
                }
                if( m_southWestView.isActive() ) {
                	m_southWestView.setActive(false);
                	m_westView.setActive(true);
                }
                destroyView(m_southEastView);
                destroyView(m_southWestView);
            }                            
	        m_mainVerticalScrollBar.setModel(m_eastView.getVerticalScrollModel());
            add(E_VIEW, m_eastView.asComponent());
            add(W_VIEW, m_westView.asComponent());
            m_northWestView = null;
            m_southWestView = null;
            m_northEastView = null;
            m_southEastView = null;
        }
        // remove the split scrollbars
        remove(m_northVerticalScrollBar);
        remove(m_southVerticalScrollBar);
        m_northVerticalScrollBar.removeAdjustmentListener(this);
        m_southVerticalScrollBar.removeAdjustmentListener(this);
        // add the main scrollbar
        m_mainVerticalScrollBar.addAdjustmentListener(this);
        add(MAIN_VSCROLLBAR, m_mainVerticalScrollBar);
    }
 
    private static JScrollBar copyScrollBar(JScrollBar sb) {
        JScrollBar rc = new JScrollBar(
            sb.getOrientation(),
            sb.getValue(),
            sb.getBlockIncrement(),
            0, 0);
        rc.setModel(sb.getModel());
        return rc;
    }
		
///////////////////////////////////////////////////////////////////////////////
// ActionListener
    public void actionPerformed(ActionEvent e) {
        if( e.getSource() == m_hSplit ) {
            switch( e.getID() ) {
                case SplitControl.POSITIONACTION: {
                    if( m_hSplit.getPosition() >= m_hSplit.getMaxPosition() ) {
                        m_hSplit.setState(SplitControl.INACTIVE);
                    } else if( m_hSplit.getPosition() <= m_hSplit.getInactivePosition() ) {
                        m_hSplit.setState(SplitControl.INACTIVE);
                    } else {
                        m_hSplit.setState(SplitControl.ACTIVE);
                    }
                } break;
                case SplitControl.STATEACTION: {
                    if( m_hSplit.isActive() ) {
                        // changing horizontal to active
                        splitHorizontal();
                        giveActiveFocus();
                    } else {
                        // changing horizontal to inactive
                        combineHorizontal();
                        giveActiveFocus();
                    }
                } break;
            }
        } else if( e.getSource() == m_vSplit ) {
            switch( e.getID() ) {
                case SplitControl.POSITIONACTION: {
                    if( m_vSplit.getPosition() >= m_vSplit.getMaxPosition() ) {
                        m_vSplit.setState(SplitControl.INACTIVE);
                    } else if( m_vSplit.getPosition() <= m_vSplit.getInactivePosition() ) {
                        m_vSplit.setState(SplitControl.INACTIVE);
                    } else {
                        m_vSplit.setState(SplitControl.ACTIVE);
                    }
                } break;
                case SplitControl.STATEACTION: {
                    if( m_vSplit.isActive() ) {
                        // changing vertical to active
                        splitVertical();
                        giveActiveFocus();
                    } else {
                        // changing vertical to inactive
                        combineVertical();
                        giveActiveFocus();
                    }
                } break;
            }
        }
        validate();
    }
// ActionListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// AdjustmentListener
    public void adjustmentValueChanged(AdjustmentEvent e) {
		if( m_mainView != null )		{ m_mainView.doLayout(); }
		if( m_northView != null )		{ m_northView.doLayout(); }
		if( m_southView != null )		{ m_southView.doLayout(); }
		if( m_eastView != null )		{ m_eastView.doLayout(); }
		if( m_westView != null )		{ m_westView.doLayout(); }
		if( m_northEastView != null )	{ m_northEastView.doLayout(); }
		if( m_northWestView != null )	{ m_northWestView.doLayout(); }
		if( m_southEastView != null )	{ m_southEastView.doLayout(); }
		if( m_southWestView != null )	{ m_southWestView.doLayout(); }
        repaint();
    }
// AdjustmentListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// FocusListener
	public void focusGained(FocusEvent e) {
		if( e.getComponent() == this ) {
			giveActiveFocus();
		} else {
			if( m_mainView != null )		{ m_mainView.setActive(false); }
			if( m_northView != null )		{ m_northView.setActive(false); }
			if( m_southView != null )		{ m_southView.setActive(false); }
			if( m_eastView != null )		{ m_eastView.setActive(false); }
			if( m_westView != null )		{ m_westView.setActive(false); }
			if( m_northEastView != null )	{ m_northEastView.setActive(false); }
			if( m_northWestView != null )	{ m_northWestView.setActive(false); }
			if( m_southEastView != null )	{ m_southEastView.setActive(false); }
			if( m_southWestView != null )	{ m_southWestView.setActive(false); }

			if( m_mainView == e.getComponent() )		{ m_mainView.setActive(true); }
			if( m_northView == e.getComponent() )		{ m_northView.setActive(true); }
			if( m_southView == e.getComponent() )		{ m_southView.setActive(true); }
			if( m_eastView == e.getComponent() )		{ m_eastView.setActive(true); }
			if( m_westView == e.getComponent() )		{ m_westView.setActive(true); }
			if( m_northEastView == e.getComponent() )	{ m_northEastView.setActive(true);}
			if( m_northWestView == e.getComponent() )	{ m_northWestView.setActive(true);}
			if( m_southEastView == e.getComponent() )	{ m_southEastView.setActive(true);}
			if( m_southWestView == e.getComponent() )	{ m_southWestView.setActive(true);}
		}
		notifyActivePaneChanged();
	}
			
	public void focusLost(FocusEvent e) {
	}
// FocusListener
///////////////////////////////////////////////////////////////////////////////
    public static void main( String[] args ) {
        try {
            UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
        } catch( Exception e ) {
            System.err.println(e);
        }
        
        final JFrame f = new JFrame();
        f.getContentPane().setLayout(new BorderLayout());
        f.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    f.dispose();
                }
                
                public void windowClosed(WindowEvent e) {
                    System.exit(0);
                }
            });
            
        SplitScrollPane sp = new SplitScrollPane();
        f.getContentPane().add( sp, BorderLayout.CENTER );
            
        f.setSize(600,700);
        f.setVisible(true);
        return;
    }
}