/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.splitscrollpane;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

public class SplitScrollPaneLayout 
    implements  java.awt.LayoutManager,
                SplitScrollPaneConstants {
        
    protected Component m_northWestView = null;
    protected Component m_southWestView = null;
    protected Component m_northEastView = null;
    protected Component m_southEastView = null;
    
    protected Component m_southView = null;
    protected Component m_eastView = null;
    protected Component m_northView = null;
    protected Component m_westView = null;
    
    protected Component m_mainView = null;
    
    protected JScrollBar m_northVScrollBar = null;
    protected JScrollBar m_southVScrollBar = null;
    protected JScrollBar m_mainVScrollBar = null;
    
    protected JScrollBar m_eastHScrollBar = null;
    protected JScrollBar m_westHScrollBar = null;
    protected JScrollBar m_mainHScrollBar = null;
    
    protected SplitControl m_hSplit = null;
    protected SplitControl m_vSplit = null;
    
    protected SplitScrollPane m_scrollpane = null;
    
    public SplitScrollPaneLayout(SplitScrollPane parent) {
        m_scrollpane = parent;
    }
    
    public void addLayoutComponent(String name, Component comp) {
    
        if( name.equals(NW_VIEW) ) {
            m_northWestView = comp;
        } else if( name.equals(NE_VIEW) ) {
            m_northEastView = comp;
        } else if( name.equals(SW_VIEW) ) {
           m_southWestView  = comp;
        } else if( name.equals(SE_VIEW) ) {
            m_southEastView = comp;
        } else if( name.equals(N_VIEW) ) {
            m_northView = comp;
        } else if( name.equals(S_VIEW) ) {
            m_southView = comp;
        } else if( name.equals(E_VIEW) ) {
            m_eastView = comp;
        } else if( name.equals(W_VIEW) ) {
            m_westView = comp;
        } else if( name.equals(MAIN_VIEW) ) {
            m_mainView = comp;
        } else if( name.equals(N_VSCROLLBAR) ) {
            m_northVScrollBar = (JScrollBar)comp;
        } else if( name.equals(S_VSCROLLBAR) ) {
            m_southVScrollBar = (JScrollBar)comp;
        } else if( name.equals(MAIN_VSCROLLBAR) ) {
            m_mainVScrollBar = (JScrollBar)comp;
        } else if( name.equals(E_HSCROLLBAR) ) {
            m_eastHScrollBar = (JScrollBar)comp;
        } else if( name.equals(W_HSCROLLBAR) ) {
            m_westHScrollBar = (JScrollBar)comp;
        } else if( name.equals(MAIN_HSCROLLBAR) ) {
            m_mainHScrollBar = (JScrollBar)comp;    
        } else if( name.equals(VSPLIT) ) {
            m_vSplit = (SplitControl)comp;
        } else if( name.equals(HSPLIT) ) {
            m_hSplit = (SplitControl)comp;    
        }
        return;
    }

    public void removeLayoutComponent(Component comp) {
        if( m_northWestView == comp ) {
            m_northWestView = null;
        } else if( m_southWestView == comp ) {
            m_southWestView = null;
        } else if( m_northEastView == comp ) {
            m_northEastView = null;
        } else if( m_southEastView == comp ) {
            m_southEastView = null;
        } else if( m_southView == comp ) {
            m_southView = null;
        } else if( m_eastView == comp ) {
            m_eastView = null;
        } else if( m_northView == comp ) {
            m_northView = null;
        } else if( m_westView == comp ) {
            m_westView = null;
        } else if( m_mainView == comp ) {
            m_mainView = null;
        } else if( m_northVScrollBar == comp ) {
            m_northVScrollBar = null;
        } else if( m_southVScrollBar == comp ) {
            m_southVScrollBar = null;
        } else if( m_mainVScrollBar == comp ) {
            m_mainVScrollBar = null;
        } else if( m_eastHScrollBar == comp ) {
            m_eastHScrollBar = null;
        } else if( m_westHScrollBar == comp ) {
            m_westHScrollBar = null;
        } else if( m_mainHScrollBar == comp ) {
            m_mainHScrollBar = null;    
        } else if( m_hSplit == comp ) {
            m_hSplit = null;
        } else if( m_vSplit == comp ) {
            m_vSplit = null;    
        }
        return;
    }
    
    public Component getLayoutComponent( String name ) {
        Component rc = null;
        if( name.equals(NW_VIEW) ) {
            rc = m_northWestView;
        } else if( name.equals(NE_VIEW) ) {
            rc = m_northEastView;
        } else if( name.equals(SW_VIEW) ) {
            rc = m_southWestView;
        } else if( name.equals(SE_VIEW) ) {
            rc = m_southEastView;
        } else if( name.equals(N_VIEW) ) {
            rc = m_northView;
        } else if( name.equals(S_VIEW) ) {
            rc = m_southView;
        } else if( name.equals(E_VIEW) ) {
            rc = m_eastView;
        } else if( name.equals(W_VIEW) ) {
            rc = m_westView;
        } else if( name.equals(MAIN_VIEW) ) {
            rc = m_mainView;
        } else if( name.equals(N_VSCROLLBAR) ) {
            rc = m_northVScrollBar;
        } else if( name.equals(S_VSCROLLBAR) ) {
            rc = m_southVScrollBar;
        } else if( name.equals(MAIN_VSCROLLBAR) ) {
            rc = m_mainVScrollBar;
        } else if( name.equals(E_HSCROLLBAR) ) {
            rc = m_eastHScrollBar;
        } else if( name.equals(W_HSCROLLBAR) ) {
            rc = m_westHScrollBar;
        } else if( name.equals(MAIN_HSCROLLBAR) ) {
            rc = m_mainHScrollBar;    
        } else if( name.equals(VSPLIT) ) {
            rc = m_vSplit;
        } else if( name.equals(HSPLIT) ) {
            rc = m_hSplit;    
        }
        return rc;
    }

    public Dimension preferredLayoutSize(Container parent) {
	    return new Dimension(0, 0);
    }

    public Dimension minimumLayoutSize(Container parent) {
	    return new Dimension(0, 0);
    }

    public void layoutContainer(Container parent) {
        // set up the boundries between split panes
        int hBounds[] = {0, 0, 0, 0 };
        int vBounds[] = {0, 0, 0, 0 };
        hBounds[3] = m_scrollpane.getBounds().width;
        vBounds[3] = m_scrollpane.getBounds().height;

        // adjust for the borders
	    Insets insets = m_scrollpane.getInsets();
	    hBounds[0] = hBounds[1] = hBounds[2] = insets.left;
	    hBounds[3] -= insets.right;
	    vBounds[0] = vBounds[1] = vBounds[2] = insets.top;
	    vBounds[3] -= insets.bottom;
        
        // adjust for edge components
        int verticalWidth = getVerticalComponentsWidth();
        int horizontalHeight = getHorizontalComponentsHeight();
        vBounds[3] -= horizontalHeight;
        hBounds[3] -= verticalWidth;
        
        // set boundaries for the splitters
	    m_hSplit.setMaxPosition(hBounds[3]-SplitControl.DEFAULTSIZE);
	    m_hSplit.setActiveEdge(vBounds[0]);
	    m_hSplit.setInactiveEdge(vBounds[3]);

	    m_vSplit.setMaxPosition(vBounds[3]-SplitControl.DEFAULTSIZE);
	    m_vSplit.setActiveEdge(hBounds[0]);
	    m_vSplit.setInactiveEdge(hBounds[3]);
	    
	    // position the splitters
	    m_vSplit.updateBounds();
	    m_hSplit.updateBounds();
        
        // use the splitter positions to calculate inner boundaries
        vBounds[1] = m_vSplit.getBounds().y;
        vBounds[2] = m_vSplit.getBounds().y+m_vSplit.getBounds().height;

        hBounds[1] = m_hSplit.getBounds().x;
        hBounds[2] = m_hSplit.getBounds().x+m_hSplit.getBounds().width;
                
        // set up the views
        // undisplayed views should be null
        if( m_northWestView != null ) {
            m_northWestView.setBounds(hBounds[0], vBounds[0], hBounds[1]-hBounds[0], vBounds[1]-vBounds[0]);
        }
        if( m_southWestView != null ) {
            m_southWestView.setBounds(hBounds[0], vBounds[2], hBounds[1]-hBounds[0], vBounds[3]-vBounds[2]);
        }
        if( m_northEastView != null ) {
            m_northEastView.setBounds(hBounds[2], vBounds[0], hBounds[3]-hBounds[2], vBounds[1]-vBounds[0]);
        }
        if( m_southEastView != null ) {
            m_southEastView.setBounds(hBounds[2], vBounds[2], hBounds[3]-hBounds[2], vBounds[3]-vBounds[2]);
        }
        if( m_southView != null ) {
            m_southView.setBounds(hBounds[0], vBounds[2], hBounds[3]-hBounds[0], vBounds[3]-vBounds[2]);
        }
        if( m_eastView != null ) {
            m_eastView.setBounds(hBounds[2], vBounds[0], hBounds[3]-hBounds[2], vBounds[3]-vBounds[0]);
        }
        if( m_northView != null ) {
            m_northView.setBounds(hBounds[0], vBounds[0], hBounds[3]-hBounds[0], vBounds[1]-vBounds[0]);
        }
        if( m_westView != null ) {
            m_westView.setBounds(hBounds[0], vBounds[0], hBounds[1]-hBounds[0], vBounds[3]-vBounds[0]);
        }
        if( m_mainView != null ) {
            m_mainView.setBounds(hBounds[0], vBounds[0], hBounds[3]-hBounds[0], vBounds[3]-vBounds[0]);
        }
        
        // set up all of the scrollbars
        // undisplayed controls should be null
        if( m_northVScrollBar != null ) {
            m_northVScrollBar.setBounds(hBounds[3], vBounds[0], verticalWidth, vBounds[1]-vBounds[0]);
            setBlockIncrement(m_northVScrollBar, (SplitScrollView)m_northView, (SplitScrollView)m_northWestView);
        }                               
        if( m_southVScrollBar != null ) {
            m_southVScrollBar.setBounds(hBounds[3], vBounds[2], verticalWidth, vBounds[3]-vBounds[2]);
            setBlockIncrement(m_southVScrollBar, (SplitScrollView)m_southView, (SplitScrollView)m_southWestView);
        }
        if( m_mainVScrollBar != null ) {
            m_mainVScrollBar.setBounds(hBounds[3], vBounds[2], verticalWidth, vBounds[3]-vBounds[2]);
            setBlockIncrement(m_mainVScrollBar, (SplitScrollView)m_mainView, (SplitScrollView)m_westView);
        }
        if( m_eastHScrollBar != null ) {
            m_eastHScrollBar.setBounds(hBounds[2], vBounds[3], hBounds[3]-hBounds[2], horizontalHeight);
            setBlockIncrement(m_eastHScrollBar, (SplitScrollView)m_eastView, (SplitScrollView)m_southEastView);
        }
        if( m_westHScrollBar != null ) {
            m_westHScrollBar.setBounds(hBounds[0], vBounds[3], hBounds[1]-hBounds[0], horizontalHeight);
            setBlockIncrement(m_westHScrollBar, (SplitScrollView)m_westView, (SplitScrollView)m_southWestView);
        }
        if( m_mainHScrollBar != null ) {
            m_mainHScrollBar.setBounds(hBounds[2], vBounds[3], hBounds[3]-hBounds[2], horizontalHeight);
            setBlockIncrement(m_mainHScrollBar, (SplitScrollView)m_mainView, (SplitScrollView)m_southView);
        }        
    }    

    private int getVerticalComponentsWidth() {
        int rc = 0;
        if( m_northVScrollBar != null ) {
	        rc = Math.max(m_northVScrollBar.getPreferredSize().width, rc);
        }
        if( m_southVScrollBar != null ) {
	        rc = Math.max(m_southVScrollBar.getPreferredSize().width, rc);
        }
        if( m_mainVScrollBar != null ) {
	        rc = Math.max(m_mainVScrollBar.getPreferredSize().width, rc);
        }
        if( m_vSplit != null ) {
	        rc = Math.max(m_hSplit.getPreferredSize().width, rc);
        }
        return rc;
    }
    
    private int getHorizontalComponentsHeight() {
        int rc = 0;
        if( m_eastHScrollBar != null ) {
	        rc = Math.max(m_eastHScrollBar.getPreferredSize().height, rc);
        }
        if( m_westHScrollBar != null ) {
	        rc = Math.max(m_westHScrollBar.getPreferredSize().height, rc);
        }
        if( m_mainHScrollBar != null ) {
	        rc = Math.max(m_mainHScrollBar.getPreferredSize().height, rc);
        }
        if( m_hSplit != null ) {
	        rc = Math.max(m_hSplit.getPreferredSize().height, rc);
        }
        return rc;
    }
 
    private void setBlockIncrement( JScrollBar sb, SplitScrollView v1, SplitScrollView v2 ) {
        int inc = 10;
        if( v1 != null ) {
            if( sb.getOrientation() == SwingConstants.HORIZONTAL ) {
                inc = v1.horizontalUnits(v1.asComponent().getBounds().width);
            } else {
                inc = v1.verticalUnits(v1.asComponent().getBounds().height);
            }
        } else if( v2 != null ) {
            if( sb.getOrientation() == SwingConstants.HORIZONTAL ) {
                inc = v2.horizontalUnits(v2.asComponent().getBounds().width);
            } else {
                inc = v2.verticalUnits(v2.asComponent().getBounds().height);
            }
        }
        sb.setBlockIncrement(inc);
    }
}