/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.undo;

import gvid.undo.*;
import gvid.ui.editor.SourceCodeEditor;
import java.util.*;

public class UndoChain 
	implements gvid.undo.UndoCommand {
	
	private String m_name = "";
	private ArrayList m_cmdList = new ArrayList();
	
	public UndoChain( String name ) {
		m_name = name;
	}
	
	public UndoChain( String name, UndoCommand cmd ) {
		m_name = name;
		m_cmdList.add(cmd);
	}
	
	public java.util.List commandList() {
		return m_cmdList;
	}

    public String getName() {
        return m_name;
    }

    public void setName(String name) {
        m_name = name;
    }

    public boolean isEditCmd() {
    	boolean rc = false;
    	Iterator i = m_cmdList.iterator();
    	while( i.hasNext() ) {
    		UndoCommand cmd = (UndoCommand)i.next();
    		rc = rc || cmd.isEditCmd();
    	}
    	return rc;
    }

    public void undo(SourceCodeEditor currentEditor) {
    	for( int i = m_cmdList.size()-1; i >= 0; i-- ) {
    		UndoCommand cmd = (UndoCommand)m_cmdList.get(i);
    		cmd.undo(currentEditor);
    	}
    }

    public void redo(SourceCodeEditor currentEditor) {
    	for( int i = 0; i < m_cmdList.size(); i++ ) {
    		UndoCommand cmd = (UndoCommand)m_cmdList.get(i);
    		cmd.redo(currentEditor);
    	}
    }
    
    public void addRedo( UndoCommand cmd ) {
    	if( cmd instanceof UndoChain ) {
    		UndoChain chain = (UndoChain)cmd;
    		for( int i = 0; i < chain.m_cmdList.size(); i++ ) {
    			UndoCommand subCmd = (UndoCommand)chain.m_cmdList.get(i);
    			m_cmdList.add(subCmd);
    		}
    	} else if( cmd != null ) {
    		m_cmdList.add(cmd);
    	}
    }
    
    public boolean combineCommands( UndoCommand cmd ) {
    	// see if the last command in the chain can combine
    	boolean rc = false;
    	if( m_cmdList.size() > 0 ) {
    		UndoCommand lastCmd = (UndoCommand)m_cmdList.get(m_cmdList.size()-1);
    		rc = lastCmd.combineCommands(cmd);
    	} else {
    		// if the cmd list is empty, anything can combine into this
    		addRedo(cmd);
    		rc = true;
    	}
    	return rc;
    }
}