/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.undo;

import java.util.ArrayList;
import gvid.ui.editor.SourceCodeEditor;

public class UndoRedoManager {
    ArrayList m_undoList = new ArrayList();
    ArrayList m_redoList = new ArrayList();
    
    public UndoRedoManager() {
    }
    
    public synchronized void undo(SourceCodeEditor currentEditor) {
        if( m_undoList.size() > 0 ) {
            UndoCommand cmd = (UndoCommand)m_undoList.remove(m_undoList.size()-1);
            cmd.undo(currentEditor);
            m_redoList.add(cmd);
        }
    }
    
    public synchronized void redo(SourceCodeEditor currentEditor) {
        if( m_redoList.size() > 0 ) {
            UndoCommand cmd = (UndoCommand)m_redoList.remove(m_redoList.size()-1);
            cmd.redo(currentEditor);
            m_undoList.add(cmd);
        }
    }
    
    public synchronized void undoChanges(SourceCodeEditor currentEditor) {
        UndoCommand cmd = null;
        do {
            cmd = (UndoCommand)m_undoList.remove(m_undoList.size()-1);
            m_redoList.add(cmd);
        } while( !cmd.isEditCmd() && m_undoList.size()>0 );
        if( cmd.isEditCmd() ) {
            cmd.undo(currentEditor);        
        }
    }
    
    public synchronized void redoChanges(SourceCodeEditor currentEditor) {
        UndoCommand cmd = null;
        do {
            cmd = (UndoCommand)m_redoList.remove(m_redoList.size()-1);
            m_undoList.add(cmd);
        } while( !cmd.isEditCmd() && m_redoList.size()>0 );
        if( cmd.isEditCmd() ) {
            cmd.redo(currentEditor);        
        }
    }
    
    public int getUndoCount() {
        return m_undoList.size();
    }
    
    public int getRedoCount() {
        return m_redoList.size();
    }
    
    public synchronized void addUndo( UndoCommand cmd ) {
    		// If the list is not empty and the new command can be combined with 
    		// the last command, then do not add the new command to the list.
    		// The redo should still be cleared.
    		if( cmd != null ) {
    			if( m_undoList.isEmpty() || 
    		    	!((UndoCommand)m_undoList.get(m_undoList.size()-1)).combineCommands(cmd) ) {
        			m_undoList.add(cmd);
    			}
        		m_redoList.clear();
    		}
    }
    
    public synchronized UndoCommand popUndo() {
    	return (UndoCommand)m_undoList.remove(m_undoList.size()-1);
    }
}