/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.util;

import java.awt.*;
import java.util.*;
import java.io.*;

public class CursorManager {
	
	private static CursorManager m_singleton = null;
	public static CursorManager getInstance() {
		if( m_singleton == null ) {
			m_singleton = new CursorManager();
		}
		return m_singleton;
	}
	
	private CursorManager() {
	}
	
	private static class CursorInfo {
		public String m_cursorName;
		public String m_imageSrc;
		public Point m_hotPoint;
		public Image m_image = null;
		public CursorInfo( String name, String imgSrc, Point hotPoint ) {
			m_cursorName = name;
			m_imageSrc = imgSrc;
			m_hotPoint = hotPoint;
		}
	}
	
	private static final CursorInfo[] m_cursorInfo = {
		new CursorInfo("DRAGMOVE",		"gvid/ui/icons/DRAGMOVE.gif",		new Point(16,  8) ),
		new CursorInfo("DRAGCOPY",		"gvid/ui/icons/DRAGCOPY.gif",		new Point(16,  8) ),
		new CursorInfo("RIGHT_DEFAULT",	"gvid/ui/icons/RIGHT_DEFAULT.gif",	new Point(12,  0) ),
		new CursorInfo("HO_SPLIT",		"gvid/ui/icons/VE_SPLIT.gif",		new Point(15, 15) ),
		new CursorInfo("VE_SPLIT",		"gvid/ui/icons/HO_SPLIT.gif",		new Point(15, 15) ),
		new CursorInfo("HV_SPLIT",		"gvid/ui/icons/HV_SPLIT.gif",		new Point( 9,  9) )
	};
	
	private Hashtable m_cursorMap = new Hashtable();
	
	public Cursor getCursor( String name ) {
		return (Cursor)m_cursorMap.get(name);
	}
	
	public void loadCursors( Component c ) {
        MediaTracker tracker = new MediaTracker(c);
        
        // load the images
        for( int i=0; i<m_cursorInfo.length; i++ ) {
        	InputStream ins = getClass().getClassLoader().getResourceAsStream(m_cursorInfo[i].m_imageSrc);
        	try {
				if ( ins.available() > 0 ) {
					byte[] imageBytes = new byte[ ins.available()+1 ];
					ins.read( imageBytes );
					ins.close();
					m_cursorInfo[i].m_image = Toolkit.getDefaultToolkit().createImage( imageBytes ) ;
				}
        	} catch( IOException e ) {
        		System.err.println(e);
        	}
        	tracker.addImage(m_cursorInfo[i].m_image, i);
        }
        // wait for them to finish loading
		for( int i=0; i<m_cursorInfo.length; i++ ) {
            try {
                tracker.waitForID(i);
            } catch (InterruptedException ie) {
                ie.printStackTrace();
            }
        	Dimension bestSize = Toolkit.getDefaultToolkit().getBestCursorSize(
        		m_cursorInfo[i].m_image.getWidth(null), 
        		m_cursorInfo[i].m_image.getHeight(null) );
        	int imageWidth = m_cursorInfo[i].m_image.getWidth(null);
        	int imageHeight = m_cursorInfo[i].m_image.getHeight(null);
        	if( /*true*/ bestSize.width!=imageWidth || bestSize.height!=imageHeight ) {
        		System.err.println("Changing size of icon.");
        		Image tempImage = c.createImage(bestSize.width, bestSize.height);
        		tempImage.getGraphics().fillRect(0, 0, bestSize.width, bestSize.height);
        		tempImage.getGraphics().drawImage(
        			m_cursorInfo[i].m_image, 
        			m_cursorInfo[i].m_image.getWidth(null), 
        			m_cursorInfo[i].m_image.getHeight(null), 
        			null);
        		m_cursorInfo[i].m_image = tempImage;
        	}
		}
		// copy them into a named cursor
        for( int i=0; i<m_cursorInfo.length; i++ ) {
        	m_cursorMap.put( 
        		m_cursorInfo[i].m_cursorName, 
        		Toolkit.getDefaultToolkit().createCustomCursor(m_cursorInfo[i].m_image, m_cursorInfo[i].m_hotPoint, m_cursorInfo[i].m_cursorName ) );
        }
	}
}
