/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.util;

import java.util.*;

public class NotifyingListIterator implements java.util.ListIterator {
	private ListIterator m_iter = null;
	
	public NotifyingListIterator() {
	}
	
	public NotifyingListIterator( ListIterator iter ) {
		m_iter = iter;
	}
	
	public void setListIterator( ListIterator iter ) {
		ListIterator oldIter = m_iter;
		m_iter = iter;
		if( oldIter != iter ) {
			notifyListIteratorChanged();
		}
	}
	
	private Vector m_listeners = new Vector();
	public void addListIteratorListener( ListIteratorListener l ) {
		m_listeners.add(l);
	}
	
	public void removeListIteratorListener( ListIteratorListener l ) {
		m_listeners.remove(l);
	}
	
	private void notifyListIteratorChanged() {
		Iterator iter = m_listeners.iterator();
		while( iter.hasNext() ) {
			ListIteratorListener l = (ListIteratorListener)iter.next();
			l.listIteratorChanged(this);
		}
	}
	
    public void add(Object o) {
		if( m_iter != null ) {
			m_iter.add(o);
			notifyListIteratorChanged();
		}
    }

    public boolean hasNext() {
    	boolean rc = false;
		if( m_iter != null ) {
			rc = m_iter.hasNext();
		}
        return rc;
    }

    public boolean hasPrevious() {
    	boolean rc = false;
		if( m_iter != null ) {
			rc = m_iter.hasPrevious();
		}
        return rc;
    }

    public Object next() {
    	Object rc = null;
		if( m_iter != null ) {
			rc = m_iter.next();
		}
        return rc;
    }

    public int nextIndex() {
    	int rc = 0;
		if( m_iter != null ) {
			rc = m_iter.nextIndex();
		}
        return rc;
    }

    public Object previous() {
    	Object rc = null;
		if( m_iter != null ) {
			rc = m_iter.previous();
		}
        return rc;
    }

    public int previousIndex() {
    	int rc = 0;
		if( m_iter != null ) {
			rc = m_iter.previousIndex();
		}
        return rc;
    }

    public void remove() {
		if( m_iter != null ) {
			m_iter.remove();
			notifyListIteratorChanged();
		}
    }

    public void set(Object o) {
		if( m_iter != null ) {			
			m_iter.set(o);
			notifyListIteratorChanged();
		}
    }
}